# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Platform Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model
from ....core import StrEnum

from ..models.transaction_amount_details import TransactionAmountDetails


class WalletActionEnum(StrEnum):
    CREDIT = "CREDIT"
    DEBIT = "DEBIT"
    PAYMENT = "PAYMENT"


class WalletTransactionInfo(Model):
    """Wallet transaction info (WalletTransactionInfo)

    Properties:
        amount: (amount) REQUIRED int

        created_at: (createdAt) REQUIRED str

        currency_code: (currencyCode) REQUIRED str

        namespace: (namespace) REQUIRED str

        operator: (operator) REQUIRED str

        updated_at: (updatedAt) REQUIRED str

        user_id: (userId) REQUIRED str

        balance_source: (balanceSource) OPTIONAL str

        metadata: (metadata) OPTIONAL Dict[str, Any]

        reason: (reason) OPTIONAL str

        transaction_amount_details: (transactionAmountDetails) OPTIONAL List[TransactionAmountDetails]

        wallet_action: (walletAction) OPTIONAL Union[str, WalletActionEnum]
    """

    # region fields

    amount: int  # REQUIRED
    created_at: str  # REQUIRED
    currency_code: str  # REQUIRED
    namespace: str  # REQUIRED
    operator: str  # REQUIRED
    updated_at: str  # REQUIRED
    user_id: str  # REQUIRED
    balance_source: str  # OPTIONAL
    metadata: Dict[str, Any]  # OPTIONAL
    reason: str  # OPTIONAL
    transaction_amount_details: List[TransactionAmountDetails]  # OPTIONAL
    wallet_action: Union[str, WalletActionEnum]  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_amount(self, value: int) -> WalletTransactionInfo:
        self.amount = value
        return self

    def with_created_at(self, value: str) -> WalletTransactionInfo:
        self.created_at = value
        return self

    def with_currency_code(self, value: str) -> WalletTransactionInfo:
        self.currency_code = value
        return self

    def with_namespace(self, value: str) -> WalletTransactionInfo:
        self.namespace = value
        return self

    def with_operator(self, value: str) -> WalletTransactionInfo:
        self.operator = value
        return self

    def with_updated_at(self, value: str) -> WalletTransactionInfo:
        self.updated_at = value
        return self

    def with_user_id(self, value: str) -> WalletTransactionInfo:
        self.user_id = value
        return self

    def with_balance_source(self, value: str) -> WalletTransactionInfo:
        self.balance_source = value
        return self

    def with_metadata(self, value: Dict[str, Any]) -> WalletTransactionInfo:
        self.metadata = value
        return self

    def with_reason(self, value: str) -> WalletTransactionInfo:
        self.reason = value
        return self

    def with_transaction_amount_details(
        self, value: List[TransactionAmountDetails]
    ) -> WalletTransactionInfo:
        self.transaction_amount_details = value
        return self

    def with_wallet_action(
        self, value: Union[str, WalletActionEnum]
    ) -> WalletTransactionInfo:
        self.wallet_action = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "amount"):
            result["amount"] = int(self.amount)
        elif include_empty:
            result["amount"] = 0
        if hasattr(self, "created_at"):
            result["createdAt"] = str(self.created_at)
        elif include_empty:
            result["createdAt"] = ""
        if hasattr(self, "currency_code"):
            result["currencyCode"] = str(self.currency_code)
        elif include_empty:
            result["currencyCode"] = ""
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "operator"):
            result["operator"] = str(self.operator)
        elif include_empty:
            result["operator"] = ""
        if hasattr(self, "updated_at"):
            result["updatedAt"] = str(self.updated_at)
        elif include_empty:
            result["updatedAt"] = ""
        if hasattr(self, "user_id"):
            result["userId"] = str(self.user_id)
        elif include_empty:
            result["userId"] = ""
        if hasattr(self, "balance_source"):
            result["balanceSource"] = str(self.balance_source)
        elif include_empty:
            result["balanceSource"] = ""
        if hasattr(self, "metadata"):
            result["metadata"] = {str(k0): v0 for k0, v0 in self.metadata.items()}
        elif include_empty:
            result["metadata"] = {}
        if hasattr(self, "reason"):
            result["reason"] = str(self.reason)
        elif include_empty:
            result["reason"] = ""
        if hasattr(self, "transaction_amount_details"):
            result["transactionAmountDetails"] = [
                i0.to_dict(include_empty=include_empty)
                for i0 in self.transaction_amount_details
            ]
        elif include_empty:
            result["transactionAmountDetails"] = []
        if hasattr(self, "wallet_action"):
            result["walletAction"] = str(self.wallet_action)
        elif include_empty:
            result["walletAction"] = Union[str, WalletActionEnum]()
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        amount: int,
        created_at: str,
        currency_code: str,
        namespace: str,
        operator: str,
        updated_at: str,
        user_id: str,
        balance_source: Optional[str] = None,
        metadata: Optional[Dict[str, Any]] = None,
        reason: Optional[str] = None,
        transaction_amount_details: Optional[List[TransactionAmountDetails]] = None,
        wallet_action: Optional[Union[str, WalletActionEnum]] = None,
        **kwargs,
    ) -> WalletTransactionInfo:
        instance = cls()
        instance.amount = amount
        instance.created_at = created_at
        instance.currency_code = currency_code
        instance.namespace = namespace
        instance.operator = operator
        instance.updated_at = updated_at
        instance.user_id = user_id
        if balance_source is not None:
            instance.balance_source = balance_source
        if metadata is not None:
            instance.metadata = metadata
        if reason is not None:
            instance.reason = reason
        if transaction_amount_details is not None:
            instance.transaction_amount_details = transaction_amount_details
        if wallet_action is not None:
            instance.wallet_action = wallet_action
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> WalletTransactionInfo:
        instance = cls()
        if not dict_:
            return instance
        if "amount" in dict_ and dict_["amount"] is not None:
            instance.amount = int(dict_["amount"])
        elif include_empty:
            instance.amount = 0
        if "createdAt" in dict_ and dict_["createdAt"] is not None:
            instance.created_at = str(dict_["createdAt"])
        elif include_empty:
            instance.created_at = ""
        if "currencyCode" in dict_ and dict_["currencyCode"] is not None:
            instance.currency_code = str(dict_["currencyCode"])
        elif include_empty:
            instance.currency_code = ""
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "operator" in dict_ and dict_["operator"] is not None:
            instance.operator = str(dict_["operator"])
        elif include_empty:
            instance.operator = ""
        if "updatedAt" in dict_ and dict_["updatedAt"] is not None:
            instance.updated_at = str(dict_["updatedAt"])
        elif include_empty:
            instance.updated_at = ""
        if "userId" in dict_ and dict_["userId"] is not None:
            instance.user_id = str(dict_["userId"])
        elif include_empty:
            instance.user_id = ""
        if "balanceSource" in dict_ and dict_["balanceSource"] is not None:
            instance.balance_source = str(dict_["balanceSource"])
        elif include_empty:
            instance.balance_source = ""
        if "metadata" in dict_ and dict_["metadata"] is not None:
            instance.metadata = {str(k0): v0 for k0, v0 in dict_["metadata"].items()}
        elif include_empty:
            instance.metadata = {}
        if "reason" in dict_ and dict_["reason"] is not None:
            instance.reason = str(dict_["reason"])
        elif include_empty:
            instance.reason = ""
        if (
            "transactionAmountDetails" in dict_
            and dict_["transactionAmountDetails"] is not None
        ):
            instance.transaction_amount_details = [
                TransactionAmountDetails.create_from_dict(
                    i0, include_empty=include_empty
                )
                for i0 in dict_["transactionAmountDetails"]
            ]
        elif include_empty:
            instance.transaction_amount_details = []
        if "walletAction" in dict_ and dict_["walletAction"] is not None:
            instance.wallet_action = str(dict_["walletAction"])
        elif include_empty:
            instance.wallet_action = Union[str, WalletActionEnum]()
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, WalletTransactionInfo]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[WalletTransactionInfo]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        WalletTransactionInfo,
        List[WalletTransactionInfo],
        Dict[Any, WalletTransactionInfo],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "amount": "amount",
            "createdAt": "created_at",
            "currencyCode": "currency_code",
            "namespace": "namespace",
            "operator": "operator",
            "updatedAt": "updated_at",
            "userId": "user_id",
            "balanceSource": "balance_source",
            "metadata": "metadata",
            "reason": "reason",
            "transactionAmountDetails": "transaction_amount_details",
            "walletAction": "wallet_action",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "amount": True,
            "createdAt": True,
            "currencyCode": True,
            "namespace": True,
            "operator": True,
            "updatedAt": True,
            "userId": True,
            "balanceSource": False,
            "metadata": False,
            "reason": False,
            "transactionAmountDetails": False,
            "walletAction": False,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "walletAction": ["CREDIT", "DEBIT", "PAYMENT"],
        }

    # endregion static methods



from typing import Optional, Tuple
from uuid import uuid4

from ._http_response import HttpResponse
from ._operation import Operation
from ._proto_http_request import ProtoHttpRequest


class FlightIdPreprocessor:
    def __init__(self, flight_id: Optional[str] = None):
        self.flight_id = flight_id or str(uuid4())

    def __call__(
        self,
        proto: ProtoHttpRequest,
        operation: Operation,
        sdk: "AccelByteSDK",
        *args,
        **kwargs
    ) -> Tuple[ProtoHttpRequest, Optional[HttpResponse]]:
        if not proto.headers.has_flight_id():
            flight_id = self.get_flight_id(operation=operation, sdk=sdk, **kwargs)
            proto.headers.add_flight_id(flight_id)
        return proto, None

    def get_flight_id(self, operation: Operation, sdk: "AccelByteSDK", **kwargs) -> str:
        kwargs_flight_id = kwargs.get("x_flight_id", None)
        if kwargs_flight_id and isinstance(kwargs_flight_id, str):
            return kwargs_flight_id

        if operation.x_flight_id:
            return operation.x_flight_id

        sdk_flight_id = sdk.get_flight_id()
        if sdk_flight_id and isinstance(sdk_flight_id, str):
            return sdk_flight_id

        return self.flight_id


FLIGHT_ID_PREPROCESSOR = FlightIdPreprocessor()


def add_flight_id(
    proto: ProtoHttpRequest, operation: Operation, sdk: "AccelByteSDK", *args, **kwargs
) -> Tuple[ProtoHttpRequest, Optional[HttpResponse]]:
    return FLIGHT_ID_PREPROCESSOR(
        proto=proto, operation=operation, sdk=sdk, *args, **kwargs
    )


def get_default_flight_id() -> str:
    return FLIGHT_ID_PREPROCESSOR.flight_id


def set_default_flight_id(flight_id: str) -> None:
    FLIGHT_ID_PREPROCESSOR.flight_id = flight_id


def update_default_flight_id(flight_id: str) -> None:
    set_default_flight_id(flight_id=flight_id)


__all__ = [
    "add_flight_id",
    "get_default_flight_id",
    "set_default_flight_id",
    "update_default_flight_id",
]



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import ErrorEntity
from ..models import RetrievePolicyPublicResponse
from ..models import RetrievePolicyResponse
from ..models import UpdatePolicyRequest

from ..operations.policies import RetrieveCountryListWithPolicies
from ..operations.policies import RetrieveLatestPolicies
from ..operations.policies import RetrieveLatestPoliciesPolicyTypeEnum
from ..operations.policies import RetrieveLatestPoliciesByNamespaceAndCountryPublic
from ..operations.policies import (
    RetrieveLatestPoliciesByNamespaceAndCountryPublicPolicyTypeEnum,
)
from ..operations.policies import RetrieveLatestPoliciesPublic
from ..operations.policies import RetrieveLatestPoliciesPublicPolicyTypeEnum
from ..operations.policies import RetrievePolicies
from ..operations.policies import SetDefaultPolicy2
from ..operations.policies import UpdatePolicy


@same_doc_as(RetrieveCountryListWithPolicies)
def retrieve_country_list_with_policies(
    x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    """Retrieve List of Countries that have Active Legal Policies (retrieveCountryListWithPolicies)

    Retrieve List of Countries that have Active and Visible Legal Policies.

    Properties:
        url: /agreement/public/policies/countries/list

        method: GET

        tags: ["Policies"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

    Responses:
        200: OK - List[str] (successful operation)
    """
    request = RetrieveCountryListWithPolicies.create()
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(RetrieveCountryListWithPolicies)
async def retrieve_country_list_with_policies_async(
    x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    """Retrieve List of Countries that have Active Legal Policies (retrieveCountryListWithPolicies)

    Retrieve List of Countries that have Active and Visible Legal Policies.

    Properties:
        url: /agreement/public/policies/countries/list

        method: GET

        tags: ["Policies"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

    Responses:
        200: OK - List[str] (successful operation)
    """
    request = RetrieveCountryListWithPolicies.create()
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(RetrieveLatestPolicies)
def retrieve_latest_policies(
    country_code: str,
    default_on_empty: Optional[bool] = None,
    policy_type: Optional[Union[str, RetrieveLatestPoliciesPolicyTypeEnum]] = None,
    tags: Optional[str] = None,
    visible_only: Optional[bool] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Retrieve Latest Policies by Country (retrieveLatestPolicies)

    Retrieve all active latest policies based on a country.
    Other detail info:

      * Leave the policyType empty if you want to be responded with all policy type
      *  Fill the tags if you want to filter the responded policy by tags
      *  Fill the defaultOnEmpty with true if you want to be responded with default country-specific policy if your requested country is not exist

    Properties:
        url: /agreement/public/policies/countries/{countryCode}

        method: GET

        tags: ["Policies"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        country_code: (countryCode) REQUIRED str in path

        default_on_empty: (defaultOnEmpty) OPTIONAL bool in query

        policy_type: (policyType) OPTIONAL Union[str, PolicyTypeEnum] in query

        tags: (tags) OPTIONAL str in query

        visible_only: (visibleOnly) OPTIONAL bool in query

    Responses:
        200: OK - List[RetrievePolicyPublicResponse] (successful operation)
    """
    request = RetrieveLatestPolicies.create(
        country_code=country_code,
        default_on_empty=default_on_empty,
        policy_type=policy_type,
        tags=tags,
        visible_only=visible_only,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(RetrieveLatestPolicies)
async def retrieve_latest_policies_async(
    country_code: str,
    default_on_empty: Optional[bool] = None,
    policy_type: Optional[Union[str, RetrieveLatestPoliciesPolicyTypeEnum]] = None,
    tags: Optional[str] = None,
    visible_only: Optional[bool] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Retrieve Latest Policies by Country (retrieveLatestPolicies)

    Retrieve all active latest policies based on a country.
    Other detail info:

      * Leave the policyType empty if you want to be responded with all policy type
      *  Fill the tags if you want to filter the responded policy by tags
      *  Fill the defaultOnEmpty with true if you want to be responded with default country-specific policy if your requested country is not exist

    Properties:
        url: /agreement/public/policies/countries/{countryCode}

        method: GET

        tags: ["Policies"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        country_code: (countryCode) REQUIRED str in path

        default_on_empty: (defaultOnEmpty) OPTIONAL bool in query

        policy_type: (policyType) OPTIONAL Union[str, PolicyTypeEnum] in query

        tags: (tags) OPTIONAL str in query

        visible_only: (visibleOnly) OPTIONAL bool in query

    Responses:
        200: OK - List[RetrievePolicyPublicResponse] (successful operation)
    """
    request = RetrieveLatestPolicies.create(
        country_code=country_code,
        default_on_empty=default_on_empty,
        policy_type=policy_type,
        tags=tags,
        visible_only=visible_only,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(RetrieveLatestPoliciesByNamespaceAndCountryPublic)
def retrieve_latest_policies_by_namespace_and_country_public(
    country_code: str,
    always_include_default: Optional[bool] = None,
    default_on_empty: Optional[bool] = None,
    policy_type: Optional[
        Union[str, RetrieveLatestPoliciesByNamespaceAndCountryPublicPolicyTypeEnum]
    ] = None,
    tags: Optional[str] = None,
    visible_only: Optional[bool] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Retrieve Latest Policies by Namespace and Country (retrieveLatestPoliciesByNamespaceAndCountryPublic)

    Retrieve all active latest policies based on a namespace and country.
    Other detail info:

      * Leave the policyType empty if you want to be responded with all policy type
      *  Fill the tags if you want to filter the responded policy by tags
      *  Fill the defaultOnEmpty with true if you want to be responded with default country-specific policy if your requested country is not exist
      *  Fill the alwaysIncludeDefault with true if you want to be responded with always include default policy. If there are duplicate policies (default policies and country specific policies with same base policy) it'll include policy with same country code, for example:
        * Document 1 (default): Region US (default), UA
        * Document 2 (default): Region US (default)
        * Document 3 (default): Region US (default)
        * User: Region UA
        * Query: alwaysIncludeDefault: true
        * Response: Document 1 (UA), Document 2 (US), Document 3 (US)

    Properties:
        url: /agreement/public/policies/namespaces/{namespace}/countries/{countryCode}

        method: GET

        tags: ["Policies"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        country_code: (countryCode) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        always_include_default: (alwaysIncludeDefault) OPTIONAL bool in query

        default_on_empty: (defaultOnEmpty) OPTIONAL bool in query

        policy_type: (policyType) OPTIONAL Union[str, PolicyTypeEnum] in query

        tags: (tags) OPTIONAL str in query

        visible_only: (visibleOnly) OPTIONAL bool in query

    Responses:
        200: OK - List[RetrievePolicyPublicResponse] (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = RetrieveLatestPoliciesByNamespaceAndCountryPublic.create(
        country_code=country_code,
        always_include_default=always_include_default,
        default_on_empty=default_on_empty,
        policy_type=policy_type,
        tags=tags,
        visible_only=visible_only,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(RetrieveLatestPoliciesByNamespaceAndCountryPublic)
async def retrieve_latest_policies_by_namespace_and_country_public_async(
    country_code: str,
    always_include_default: Optional[bool] = None,
    default_on_empty: Optional[bool] = None,
    policy_type: Optional[
        Union[str, RetrieveLatestPoliciesByNamespaceAndCountryPublicPolicyTypeEnum]
    ] = None,
    tags: Optional[str] = None,
    visible_only: Optional[bool] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Retrieve Latest Policies by Namespace and Country (retrieveLatestPoliciesByNamespaceAndCountryPublic)

    Retrieve all active latest policies based on a namespace and country.
    Other detail info:

      * Leave the policyType empty if you want to be responded with all policy type
      *  Fill the tags if you want to filter the responded policy by tags
      *  Fill the defaultOnEmpty with true if you want to be responded with default country-specific policy if your requested country is not exist
      *  Fill the alwaysIncludeDefault with true if you want to be responded with always include default policy. If there are duplicate policies (default policies and country specific policies with same base policy) it'll include policy with same country code, for example:
        * Document 1 (default): Region US (default), UA
        * Document 2 (default): Region US (default)
        * Document 3 (default): Region US (default)
        * User: Region UA
        * Query: alwaysIncludeDefault: true
        * Response: Document 1 (UA), Document 2 (US), Document 3 (US)

    Properties:
        url: /agreement/public/policies/namespaces/{namespace}/countries/{countryCode}

        method: GET

        tags: ["Policies"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        country_code: (countryCode) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        always_include_default: (alwaysIncludeDefault) OPTIONAL bool in query

        default_on_empty: (defaultOnEmpty) OPTIONAL bool in query

        policy_type: (policyType) OPTIONAL Union[str, PolicyTypeEnum] in query

        tags: (tags) OPTIONAL str in query

        visible_only: (visibleOnly) OPTIONAL bool in query

    Responses:
        200: OK - List[RetrievePolicyPublicResponse] (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = RetrieveLatestPoliciesByNamespaceAndCountryPublic.create(
        country_code=country_code,
        always_include_default=always_include_default,
        default_on_empty=default_on_empty,
        policy_type=policy_type,
        tags=tags,
        visible_only=visible_only,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(RetrieveLatestPoliciesPublic)
def retrieve_latest_policies_public(
    always_include_default: Optional[bool] = None,
    default_on_empty: Optional[bool] = None,
    policy_type: Optional[
        Union[str, RetrieveLatestPoliciesPublicPolicyTypeEnum]
    ] = None,
    tags: Optional[str] = None,
    visible_only: Optional[bool] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Retrieve Latest Policies by Namespace and Country (retrieveLatestPoliciesPublic)

    Retrieve all active latest policies based on a namespace and country. The country will be read from user token.
    Other detail info:

      * Leave the policyType empty if you want to be responded with all policy type
      *  Fill the tags if you want to filter the responded policy by tags
      *  Fill the defaultOnEmpty with true if you want to be responded with default country-specific policy if your requested country is not exist
      *  Fill the alwaysIncludeDefault with true if you want to be responded with always include default policy. If there are duplicate policies (default policies and country specific policies with same base policy) it'll include policy with same country code, for example:
        * Document 1 (default): Region US (default), UA
        * Document 2 (default): Region US (default)
        * Document 3 (default): Region US (default)
        * User: Region UA
        * Query: alwaysIncludeDefault: true
        * Response: Document 1 (UA), Document 2 (US), Document 3 (US)

    Properties:
        url: /agreement/public/policies/namespaces/{namespace}

        method: GET

        tags: ["Policies"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        always_include_default: (alwaysIncludeDefault) OPTIONAL bool in query

        default_on_empty: (defaultOnEmpty) OPTIONAL bool in query

        policy_type: (policyType) OPTIONAL Union[str, PolicyTypeEnum] in query

        tags: (tags) OPTIONAL str in query

        visible_only: (visibleOnly) OPTIONAL bool in query

    Responses:
        200: OK - List[RetrievePolicyPublicResponse] (successful operation)

        404: Not Found - ErrorEntity (40040: errors.net.accelbyte.platform.legal.user_has_no_country)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = RetrieveLatestPoliciesPublic.create(
        always_include_default=always_include_default,
        default_on_empty=default_on_empty,
        policy_type=policy_type,
        tags=tags,
        visible_only=visible_only,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(RetrieveLatestPoliciesPublic)
async def retrieve_latest_policies_public_async(
    always_include_default: Optional[bool] = None,
    default_on_empty: Optional[bool] = None,
    policy_type: Optional[
        Union[str, RetrieveLatestPoliciesPublicPolicyTypeEnum]
    ] = None,
    tags: Optional[str] = None,
    visible_only: Optional[bool] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Retrieve Latest Policies by Namespace and Country (retrieveLatestPoliciesPublic)

    Retrieve all active latest policies based on a namespace and country. The country will be read from user token.
    Other detail info:

      * Leave the policyType empty if you want to be responded with all policy type
      *  Fill the tags if you want to filter the responded policy by tags
      *  Fill the defaultOnEmpty with true if you want to be responded with default country-specific policy if your requested country is not exist
      *  Fill the alwaysIncludeDefault with true if you want to be responded with always include default policy. If there are duplicate policies (default policies and country specific policies with same base policy) it'll include policy with same country code, for example:
        * Document 1 (default): Region US (default), UA
        * Document 2 (default): Region US (default)
        * Document 3 (default): Region US (default)
        * User: Region UA
        * Query: alwaysIncludeDefault: true
        * Response: Document 1 (UA), Document 2 (US), Document 3 (US)

    Properties:
        url: /agreement/public/policies/namespaces/{namespace}

        method: GET

        tags: ["Policies"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        always_include_default: (alwaysIncludeDefault) OPTIONAL bool in query

        default_on_empty: (defaultOnEmpty) OPTIONAL bool in query

        policy_type: (policyType) OPTIONAL Union[str, PolicyTypeEnum] in query

        tags: (tags) OPTIONAL str in query

        visible_only: (visibleOnly) OPTIONAL bool in query

    Responses:
        200: OK - List[RetrievePolicyPublicResponse] (successful operation)

        404: Not Found - ErrorEntity (40040: errors.net.accelbyte.platform.legal.user_has_no_country)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = RetrieveLatestPoliciesPublic.create(
        always_include_default=always_include_default,
        default_on_empty=default_on_empty,
        policy_type=policy_type,
        tags=tags,
        visible_only=visible_only,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(RetrievePolicies)
def retrieve_policies(
    country_code: str, x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    """Retrieve Policies by Country (retrievePolicies)

    Retrieve all active policies based on a country.

    Properties:
        url: /agreement/admin/policies/countries/{countryCode}

        method: GET

        tags: ["Policies"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        country_code: (countryCode) REQUIRED str in path

    Responses:
        200: OK - List[RetrievePolicyResponse] (successful operation)
    """
    request = RetrievePolicies.create(
        country_code=country_code,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(RetrievePolicies)
async def retrieve_policies_async(
    country_code: str, x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    """Retrieve Policies by Country (retrievePolicies)

    Retrieve all active policies based on a country.

    Properties:
        url: /agreement/admin/policies/countries/{countryCode}

        method: GET

        tags: ["Policies"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        country_code: (countryCode) REQUIRED str in path

    Responses:
        200: OK - List[RetrievePolicyResponse] (successful operation)
    """
    request = RetrievePolicies.create(
        country_code=country_code,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(SetDefaultPolicy2)
def set_default_policy_2(
    policy_id: str, x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    """Set Default Policy (setDefaultPolicy_2)

    Update a policy to be the default.

    Properties:
        url: /agreement/admin/policies/{policyId}/default

        method: PATCH

        tags: ["Policies"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        policy_id: (policyId) REQUIRED str in path

    Responses:
        200: OK - (operation successful)

        400: Bad Request - ErrorEntity (40033: errors.net.accelbyte.platform.legal.invalid_policy_id)
    """
    request = SetDefaultPolicy2.create(
        policy_id=policy_id,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(SetDefaultPolicy2)
async def set_default_policy_2_async(
    policy_id: str, x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    """Set Default Policy (setDefaultPolicy_2)

    Update a policy to be the default.

    Properties:
        url: /agreement/admin/policies/{policyId}/default

        method: PATCH

        tags: ["Policies"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        policy_id: (policyId) REQUIRED str in path

    Responses:
        200: OK - (operation successful)

        400: Bad Request - ErrorEntity (40033: errors.net.accelbyte.platform.legal.invalid_policy_id)
    """
    request = SetDefaultPolicy2.create(
        policy_id=policy_id,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(UpdatePolicy)
def update_policy(
    policy_id: str,
    body: Optional[UpdatePolicyRequest] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update Country-Specific Policy (updatePolicy)

    Update country-specific policy.

    Properties:
        url: /agreement/admin/policies/{policyId}

        method: PATCH

        tags: ["Policies"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) OPTIONAL UpdatePolicyRequest in body

        policy_id: (policyId) REQUIRED str in path

    Responses:
        200: OK - (operation successful)

        400: Bad Request - ErrorEntity (40033: errors.net.accelbyte.platform.legal.invalid_policy_id)
    """
    request = UpdatePolicy.create(
        policy_id=policy_id,
        body=body,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(UpdatePolicy)
async def update_policy_async(
    policy_id: str,
    body: Optional[UpdatePolicyRequest] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update Country-Specific Policy (updatePolicy)

    Update country-specific policy.

    Properties:
        url: /agreement/admin/policies/{policyId}

        method: PATCH

        tags: ["Policies"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) OPTIONAL UpdatePolicyRequest in body

        policy_id: (policyId) REQUIRED str in path

    Responses:
        200: OK - (operation successful)

        400: Bad Request - ErrorEntity (40033: errors.net.accelbyte.platform.legal.invalid_policy_id)
    """
    request = UpdatePolicy.create(
        policy_id=policy_id,
        body=body,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Session History Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class ApimodelsTicketObservabilityHistory(Model):
    """Apimodels ticket observability history (apimodels.TicketObservabilityHistory)

    Properties:
        action: (Action) REQUIRED str

        expired_at: (ExpiredAt) REQUIRED str

        id_: (ID) REQUIRED str

        party_id: (party_id) REQUIRED str

        payload: (Payload) REQUIRED Dict[str, Any]

        pod_name: (PodName) REQUIRED str

        tick_id: (TickID) REQUIRED int

        ticket_id: (ticket_id) REQUIRED str

        timestamp: (Timestamp) REQUIRED str
    """

    # region fields

    action: str  # REQUIRED
    expired_at: str  # REQUIRED
    id_: str  # REQUIRED
    party_id: str  # REQUIRED
    payload: Dict[str, Any]  # REQUIRED
    pod_name: str  # REQUIRED
    tick_id: int  # REQUIRED
    ticket_id: str  # REQUIRED
    timestamp: str  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_action(self, value: str) -> ApimodelsTicketObservabilityHistory:
        self.action = value
        return self

    def with_expired_at(self, value: str) -> ApimodelsTicketObservabilityHistory:
        self.expired_at = value
        return self

    def with_id(self, value: str) -> ApimodelsTicketObservabilityHistory:
        self.id_ = value
        return self

    def with_party_id(self, value: str) -> ApimodelsTicketObservabilityHistory:
        self.party_id = value
        return self

    def with_payload(
        self, value: Dict[str, Any]
    ) -> ApimodelsTicketObservabilityHistory:
        self.payload = value
        return self

    def with_pod_name(self, value: str) -> ApimodelsTicketObservabilityHistory:
        self.pod_name = value
        return self

    def with_tick_id(self, value: int) -> ApimodelsTicketObservabilityHistory:
        self.tick_id = value
        return self

    def with_ticket_id(self, value: str) -> ApimodelsTicketObservabilityHistory:
        self.ticket_id = value
        return self

    def with_timestamp(self, value: str) -> ApimodelsTicketObservabilityHistory:
        self.timestamp = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "action"):
            result["Action"] = str(self.action)
        elif include_empty:
            result["Action"] = ""
        if hasattr(self, "expired_at"):
            result["ExpiredAt"] = str(self.expired_at)
        elif include_empty:
            result["ExpiredAt"] = ""
        if hasattr(self, "id_"):
            result["ID"] = str(self.id_)
        elif include_empty:
            result["ID"] = ""
        if hasattr(self, "party_id"):
            result["party_id"] = str(self.party_id)
        elif include_empty:
            result["party_id"] = ""
        if hasattr(self, "payload"):
            result["Payload"] = {str(k0): v0 for k0, v0 in self.payload.items()}
        elif include_empty:
            result["Payload"] = {}
        if hasattr(self, "pod_name"):
            result["PodName"] = str(self.pod_name)
        elif include_empty:
            result["PodName"] = ""
        if hasattr(self, "tick_id"):
            result["TickID"] = int(self.tick_id)
        elif include_empty:
            result["TickID"] = 0
        if hasattr(self, "ticket_id"):
            result["ticket_id"] = str(self.ticket_id)
        elif include_empty:
            result["ticket_id"] = ""
        if hasattr(self, "timestamp"):
            result["Timestamp"] = str(self.timestamp)
        elif include_empty:
            result["Timestamp"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        action: str,
        expired_at: str,
        id_: str,
        party_id: str,
        payload: Dict[str, Any],
        pod_name: str,
        tick_id: int,
        ticket_id: str,
        timestamp: str,
        **kwargs,
    ) -> ApimodelsTicketObservabilityHistory:
        instance = cls()
        instance.action = action
        instance.expired_at = expired_at
        instance.id_ = id_
        instance.party_id = party_id
        instance.payload = payload
        instance.pod_name = pod_name
        instance.tick_id = tick_id
        instance.ticket_id = ticket_id
        instance.timestamp = timestamp
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ApimodelsTicketObservabilityHistory:
        instance = cls()
        if not dict_:
            return instance
        if "Action" in dict_ and dict_["Action"] is not None:
            instance.action = str(dict_["Action"])
        elif include_empty:
            instance.action = ""
        if "ExpiredAt" in dict_ and dict_["ExpiredAt"] is not None:
            instance.expired_at = str(dict_["ExpiredAt"])
        elif include_empty:
            instance.expired_at = ""
        if "ID" in dict_ and dict_["ID"] is not None:
            instance.id_ = str(dict_["ID"])
        elif include_empty:
            instance.id_ = ""
        if "party_id" in dict_ and dict_["party_id"] is not None:
            instance.party_id = str(dict_["party_id"])
        elif include_empty:
            instance.party_id = ""
        if "Payload" in dict_ and dict_["Payload"] is not None:
            instance.payload = {str(k0): v0 for k0, v0 in dict_["Payload"].items()}
        elif include_empty:
            instance.payload = {}
        if "PodName" in dict_ and dict_["PodName"] is not None:
            instance.pod_name = str(dict_["PodName"])
        elif include_empty:
            instance.pod_name = ""
        if "TickID" in dict_ and dict_["TickID"] is not None:
            instance.tick_id = int(dict_["TickID"])
        elif include_empty:
            instance.tick_id = 0
        if "ticket_id" in dict_ and dict_["ticket_id"] is not None:
            instance.ticket_id = str(dict_["ticket_id"])
        elif include_empty:
            instance.ticket_id = ""
        if "Timestamp" in dict_ and dict_["Timestamp"] is not None:
            instance.timestamp = str(dict_["Timestamp"])
        elif include_empty:
            instance.timestamp = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ApimodelsTicketObservabilityHistory]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ApimodelsTicketObservabilityHistory]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ApimodelsTicketObservabilityHistory,
        List[ApimodelsTicketObservabilityHistory],
        Dict[Any, ApimodelsTicketObservabilityHistory],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "Action": "action",
            "ExpiredAt": "expired_at",
            "ID": "id_",
            "party_id": "party_id",
            "Payload": "payload",
            "PodName": "pod_name",
            "TickID": "tick_id",
            "ticket_id": "ticket_id",
            "Timestamp": "timestamp",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "Action": True,
            "ExpiredAt": True,
            "ID": True,
            "party_id": True,
            "Payload": True,
            "PodName": True,
            "TickID": True,
            "ticket_id": True,
            "Timestamp": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the AccelByte Gaming Services Session Service."""

__version__ = "3.17.1"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .allocs_handler import AllocsHandler
from .block_handler import BlockHandler
from .cmdline_handler import CmdlineHandler
from .get_healthcheck_info import GetHealthcheckInfo
from .get_healthcheck_info_v1 import GetHealthcheckInfoV1
from .goroutine_handler import GoroutineHandler
from .heap_handler import HeapHandler
from .mutex_handler import MutexHandler
from .pprof_handler import PprofHandler
from .profile import Profile
from .symbol_handler import SymbolHandler
from .threadcreate_handler import ThreadcreateHandler
from .trace_handler import TraceHandler



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Ugc Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from .utils import randomize

from ..api.ugc.models import ModelsAddDownloadCountResponse
from ..api.ugc.models import ModelsAdminContentRequestV2
from ..api.ugc.models import ModelsAdminGetContentBulkRequest
from ..api.ugc.models import ModelsAdminUpdateContentRequest
from ..api.ugc.models import ModelsAdminUpdateContentRequestV2
from ..api.ugc.models import ModelsApproveStagingContentRequest
from ..api.ugc.models import ModelsChannelRequest
from ..api.ugc.models import ModelsChannelResponse
from ..api.ugc.models import ModelsConfigResponse
from ..api.ugc.models import ModelsContentDownloadResponse
from ..api.ugc.models import ModelsContentDownloadResponseV2
from ..api.ugc.models import ModelsContentDownloaderResponse
from ..api.ugc.models import ModelsContentLikeRequest
from ..api.ugc.models import ModelsContentLikeResponse
from ..api.ugc.models import ModelsContentLikersResponse
from ..api.ugc.models import ModelsContentRequestV2
from ..api.ugc.models import ModelsContentSnapshot
from ..api.ugc.models import ModelsContentVersionResponse
from ..api.ugc.models import ModelsCopyContentRequest
from ..api.ugc.models import ModelsCreateContentRequest
from ..api.ugc.models import ModelsCreateContentRequestS3
from ..api.ugc.models import ModelsCreateContentResponse
from ..api.ugc.models import ModelsCreateContentResponseV2
from ..api.ugc.models import ModelsCreateGroupRequest
from ..api.ugc.models import ModelsCreateGroupResponse
from ..api.ugc.models import ModelsCreateScreenshotRequest
from ..api.ugc.models import ModelsCreateScreenshotRequestItem
from ..api.ugc.models import ModelsCreateScreenshotResponse
from ..api.ugc.models import ModelsCreateTagRequest
from ..api.ugc.models import ModelsCreateTagResponse
from ..api.ugc.models import ModelsCreateTypeRequest
from ..api.ugc.models import ModelsCreateTypeResponse
from ..api.ugc.models import ModelsCreatorFollowState
from ..api.ugc.models import ModelsCreatorOverviewResponse
from ..api.ugc.models import ModelsCreatorResponse
from ..api.ugc.models import ModelsGenerateContentUploadURLRequest
from ..api.ugc.models import ModelsGenerateContentUploadURLResponse
from ..api.ugc.models import ModelsGetContentBulkByShareCodesRequest
from ..api.ugc.models import ModelsGetContentPreviewResponse
from ..api.ugc.models import ModelsHideContentRequest
from ..api.ugc.models import ModelsLikeState
from ..api.ugc.models import ModelsListContentVersionsResponse
from ..api.ugc.models import ModelsPaginatedContentDownloadResponse
from ..api.ugc.models import ModelsPaginatedContentDownloadResponseV2
from ..api.ugc.models import ModelsPaginatedContentDownloaderResponse
from ..api.ugc.models import ModelsPaginatedContentLikersResponse
from ..api.ugc.models import ModelsPaginatedCreatorOverviewResponse
from ..api.ugc.models import ModelsPaginatedGetChannelResponse
from ..api.ugc.models import ModelsPaginatedGetConfigsResponse
from ..api.ugc.models import ModelsPaginatedGetTagResponse
from ..api.ugc.models import ModelsPaginatedGetTypeResponse
from ..api.ugc.models import ModelsPaginatedGroupResponse
from ..api.ugc.models import ModelsPaginatedListStagingContentResponse
from ..api.ugc.models import ModelsPagingCursor
from ..api.ugc.models import ModelsPayloadURL
from ..api.ugc.models import ModelsPreviewMetadata
from ..api.ugc.models import ModelsPreviewURL
from ..api.ugc.models import ModelsPublicChannelRequest
from ..api.ugc.models import ModelsPublicCreateContentRequestS3
from ..api.ugc.models import ModelsPublicGetContentBulkRequest
from ..api.ugc.models import ModelsScreenshotResponse
from ..api.ugc.models import ModelsStagingContentResponse
from ..api.ugc.models import ModelsUpdateChannelRequest
from ..api.ugc.models import ModelsUpdateConfigRequest
from ..api.ugc.models import ModelsUpdateContentRequest
from ..api.ugc.models import ModelsUpdateContentRequestV2
from ..api.ugc.models import ModelsUpdateContentResponseV2
from ..api.ugc.models import ModelsUpdateContentShareCodeRequest
from ..api.ugc.models import ModelsUpdateFileLocationRequest
from ..api.ugc.models import ModelsUpdateScreenshot
from ..api.ugc.models import ModelsUpdateScreenshotRequest
from ..api.ugc.models import ModelsUpdateScreenshotResponse
from ..api.ugc.models import ModelsUpdateStagingContentRequest
from ..api.ugc.models import ModelsUserFollowRequest
from ..api.ugc.models import ModelsUserFollowResponse
from ..api.ugc.models import ResponseError


def create_models_add_download_count_response_example() -> (
    ModelsAddDownloadCountResponse
):
    instance = ModelsAddDownloadCountResponse()
    instance.content_id = randomize()
    return instance


def create_models_admin_content_request_v2_example() -> ModelsAdminContentRequestV2:
    instance = ModelsAdminContentRequestV2()
    instance.name = randomize()
    instance.content_type = randomize()
    instance.custom_attributes = {randomize(): randomize()}
    instance.file_extension = randomize()
    instance.share_code = randomize()
    instance.sub_type = randomize()
    instance.tags = [randomize()]
    instance.type_ = randomize()
    return instance


def create_models_admin_get_content_bulk_request_example() -> (
    ModelsAdminGetContentBulkRequest
):
    instance = ModelsAdminGetContentBulkRequest()
    instance.content_ids = [randomize()]
    return instance


def create_models_admin_update_content_request_example() -> (
    ModelsAdminUpdateContentRequest
):
    instance = ModelsAdminUpdateContentRequest()
    instance.content_type = randomize()
    instance.file_extension = randomize()
    instance.name = randomize()
    instance.payload = randomize()
    instance.preview = randomize()
    instance.sub_type = randomize()
    instance.tags = [randomize()]
    instance.type_ = randomize()
    instance.update_content_file = randomize("bool")
    instance.custom_attributes = {randomize(): randomize()}
    instance.preview_metadata = create_models_preview_metadata_example()
    instance.share_code = randomize()
    return instance


def create_models_admin_update_content_request_v2_example() -> (
    ModelsAdminUpdateContentRequestV2
):
    instance = ModelsAdminUpdateContentRequestV2()
    instance.custom_attributes = {randomize(): randomize()}
    instance.name = randomize()
    instance.share_code = randomize()
    instance.sub_type = randomize()
    instance.tags = [randomize()]
    instance.type_ = randomize()
    return instance


def create_models_approve_staging_content_request_example() -> (
    ModelsApproveStagingContentRequest
):
    instance = ModelsApproveStagingContentRequest()
    instance.approved = randomize("bool")
    instance.note = randomize()
    return instance


def create_models_channel_request_example() -> ModelsChannelRequest:
    instance = ModelsChannelRequest()
    instance.name = randomize()
    instance.id_ = randomize()
    return instance


def create_models_channel_response_example() -> ModelsChannelResponse:
    instance = ModelsChannelResponse()
    instance.id_ = randomize()
    instance.name = randomize()
    instance.namespace = randomize("slug")
    instance.user_id = randomize("uid")
    instance.parent_namespace = randomize("slug")
    return instance


def create_models_config_response_example() -> ModelsConfigResponse:
    instance = ModelsConfigResponse()
    instance.key = randomize()
    instance.namespace = randomize("slug")
    instance.value = randomize()
    return instance


def create_models_content_download_response_example() -> ModelsContentDownloadResponse:
    instance = ModelsContentDownloadResponse()
    instance.channel_id = randomize()
    instance.created_time = randomize()
    instance.creator_follow_state = create_models_creator_follow_state_example()
    instance.creator_name = randomize()
    instance.download_count = randomize("int", min_val=1, max_val=1000)
    instance.file_extension = randomize()
    instance.id_ = randomize()
    instance.is_hidden = randomize("bool")
    instance.is_official = randomize("bool")
    instance.like_count = randomize("int", min_val=1, max_val=1000)
    instance.name = randomize()
    instance.namespace = randomize("slug")
    instance.share_code = randomize()
    instance.sub_type = randomize()
    instance.tags = [randomize()]
    instance.type_ = randomize()
    instance.updated_time = randomize()
    instance.user_id = randomize("uid")
    instance.custom_attributes = {randomize(): randomize()}
    instance.groups = [randomize()]
    instance.like_state = create_models_like_state_example()
    instance.payload = randomize()
    instance.payload_url = randomize("url")
    instance.preview_url = randomize("url")
    instance.screenshots = [create_models_screenshot_response_example()]
    return instance


def create_models_content_download_response_v2_example() -> (
    ModelsContentDownloadResponseV2
):
    instance = ModelsContentDownloadResponseV2()
    instance.channel_id = randomize()
    instance.created_time = randomize()
    instance.creator_follow_state = create_models_creator_follow_state_example()
    instance.download_count = randomize("int", min_val=1, max_val=1000)
    instance.id_ = randomize()
    instance.is_hidden = randomize("bool")
    instance.is_official = randomize("bool")
    instance.like_count = randomize("int", min_val=1, max_val=1000)
    instance.name = randomize()
    instance.namespace = randomize("slug")
    instance.share_code = randomize()
    instance.tags = [randomize()]
    instance.user_id = randomize("uid")
    instance.custom_attributes = {randomize(): randomize()}
    instance.file_extension = randomize()
    instance.groups = [randomize()]
    instance.like_state = create_models_like_state_example()
    instance.payload_url = randomize("url")
    instance.screenshots = [create_models_screenshot_response_example()]
    instance.sub_type = randomize()
    instance.type_ = randomize()
    instance.updated_time = randomize()
    return instance


def create_models_content_downloader_response_example() -> (
    ModelsContentDownloaderResponse
):
    instance = ModelsContentDownloaderResponse()
    instance.content_id = randomize()
    instance.created_time = randomize("date")
    instance.downloaded_by = randomize()
    instance.namespace = randomize("slug")
    return instance


def create_models_content_like_request_example() -> ModelsContentLikeRequest:
    instance = ModelsContentLikeRequest()
    instance.like_status = randomize("bool")
    return instance


def create_models_content_like_response_example() -> ModelsContentLikeResponse:
    instance = ModelsContentLikeResponse()
    instance.content_id = randomize()
    instance.like_status = randomize("bool")
    return instance


def create_models_content_likers_response_example() -> ModelsContentLikersResponse:
    instance = ModelsContentLikersResponse()
    instance.content_id = randomize()
    instance.created_time = randomize("date")
    instance.liked_by = randomize()
    instance.namespace = randomize("slug")
    return instance


def create_models_content_request_v2_example() -> ModelsContentRequestV2:
    instance = ModelsContentRequestV2()
    instance.name = randomize()
    instance.content_type = randomize()
    instance.custom_attributes = {randomize(): randomize()}
    instance.file_extension = randomize()
    instance.sub_type = randomize()
    instance.tags = [randomize()]
    instance.type_ = randomize()
    return instance


def create_models_content_snapshot_example() -> ModelsContentSnapshot:
    instance = ModelsContentSnapshot()
    instance.file_extension = randomize()
    instance.name = randomize()
    instance.url = randomize("url")
    return instance


def create_models_content_version_response_example() -> ModelsContentVersionResponse:
    instance = ModelsContentVersionResponse()
    instance.content = create_models_content_snapshot_example()
    instance.id_ = randomize()
    instance.updated_at = randomize()
    instance.version = randomize("version")
    return instance


def create_models_copy_content_request_example() -> ModelsCopyContentRequest:
    instance = ModelsCopyContentRequest()
    instance.target_channel_id = randomize()
    instance.custom_attributes = {randomize(): randomize()}
    instance.name = randomize()
    instance.sub_type = randomize()
    instance.tags = [randomize()]
    instance.type_ = randomize()
    return instance


def create_models_create_content_request_example() -> ModelsCreateContentRequest:
    instance = ModelsCreateContentRequest()
    instance.name = randomize()
    instance.payload = randomize()
    instance.preview = randomize()
    instance.sub_type = randomize()
    instance.tags = [randomize()]
    instance.type_ = randomize()
    instance.custom_attributes = {randomize(): randomize()}
    instance.preview_metadata = create_models_preview_metadata_example()
    return instance


def create_models_create_content_request_s3_example() -> ModelsCreateContentRequestS3:
    instance = ModelsCreateContentRequestS3()
    instance.name = randomize()
    instance.preview = randomize()
    instance.sub_type = randomize()
    instance.tags = [randomize()]
    instance.type_ = randomize()
    instance.content_type = randomize()
    instance.custom_attributes = {randomize(): randomize()}
    instance.file_extension = randomize()
    instance.preview_metadata = create_models_preview_metadata_example()
    instance.share_code = randomize()
    return instance


def create_models_create_content_response_example() -> ModelsCreateContentResponse:
    instance = ModelsCreateContentResponse()
    instance.channel_id = randomize()
    instance.created_time = randomize()
    instance.creator_name = randomize()
    instance.file_extension = randomize()
    instance.id_ = randomize()
    instance.is_hidden = randomize("bool")
    instance.is_official = randomize("bool")
    instance.name = randomize()
    instance.namespace = randomize("slug")
    instance.parent_namespace = randomize("slug")
    instance.preview = randomize()
    instance.share_code = randomize()
    instance.sub_type = randomize()
    instance.tags = [randomize()]
    instance.type_ = randomize()
    instance.updated_time = randomize()
    instance.user_id = randomize("uid")
    instance.content_status = randomize()
    instance.content_type = randomize()
    instance.custom_attributes = {randomize(): randomize()}
    instance.payload_url = randomize("url")
    instance.preview_url = randomize("url")
    return instance


def create_models_create_content_response_v2_example() -> ModelsCreateContentResponseV2:
    instance = ModelsCreateContentResponseV2()
    instance.channel_id = randomize()
    instance.content_status = randomize()
    instance.created_time = randomize()
    instance.file_location = randomize()
    instance.id_ = randomize()
    instance.is_hidden = randomize("bool")
    instance.is_official = randomize("bool")
    instance.name = randomize()
    instance.namespace = randomize("slug")
    instance.parent_namespace = randomize("slug")
    instance.share_code = randomize()
    instance.tags = [randomize()]
    instance.user_id = randomize("uid")
    instance.content_type = randomize()
    instance.custom_attributes = {randomize(): randomize()}
    instance.file_extension = randomize()
    instance.payload_url = randomize("url")
    instance.sub_type = randomize()
    instance.type_ = randomize()
    instance.updated_time = randomize()
    return instance


def create_models_create_group_request_example() -> ModelsCreateGroupRequest:
    instance = ModelsCreateGroupRequest()
    instance.contents = [randomize()]
    instance.name = randomize()
    return instance


def create_models_create_group_response_example() -> ModelsCreateGroupResponse:
    instance = ModelsCreateGroupResponse()
    instance.contents = [randomize()]
    instance.created_at = randomize()
    instance.id_ = randomize()
    instance.name = randomize()
    instance.namespace = randomize("slug")
    instance.parent_namespace = randomize("slug")
    instance.user_id = randomize("uid")
    return instance


def create_models_create_screenshot_request_example() -> ModelsCreateScreenshotRequest:
    instance = ModelsCreateScreenshotRequest()
    instance.screenshots = [create_models_create_screenshot_request_item_example()]
    return instance


def create_models_create_screenshot_request_item_example() -> (
    ModelsCreateScreenshotRequestItem
):
    instance = ModelsCreateScreenshotRequestItem()
    instance.description = randomize()
    instance.file_extension = randomize()
    instance.content_type = randomize()
    return instance


def create_models_create_screenshot_response_example() -> (
    ModelsCreateScreenshotResponse
):
    instance = ModelsCreateScreenshotResponse()
    instance.screenshots = [create_models_screenshot_response_example()]
    return instance


def create_models_create_tag_request_example() -> ModelsCreateTagRequest:
    instance = ModelsCreateTagRequest()
    instance.tag = randomize()
    return instance


def create_models_create_tag_response_example() -> ModelsCreateTagResponse:
    instance = ModelsCreateTagResponse()
    instance.id_ = randomize()
    instance.namespace = randomize("slug")
    instance.tag = randomize()
    return instance


def create_models_create_type_request_example() -> ModelsCreateTypeRequest:
    instance = ModelsCreateTypeRequest()
    instance.subtype = [randomize()]
    instance.type_ = randomize()
    return instance


def create_models_create_type_response_example() -> ModelsCreateTypeResponse:
    instance = ModelsCreateTypeResponse()
    instance.id_ = randomize()
    instance.namespace = randomize("slug")
    instance.parent_namespace = randomize("slug")
    instance.subtype = [randomize()]
    instance.type_ = randomize()
    return instance


def create_models_creator_follow_state_example() -> ModelsCreatorFollowState:
    instance = ModelsCreatorFollowState()
    instance.state = randomize("bool")
    instance.user_id = randomize("uid")
    return instance


def create_models_creator_overview_response_example() -> ModelsCreatorOverviewResponse:
    instance = ModelsCreatorOverviewResponse()
    instance.follow_count = randomize("int", min_val=1, max_val=1000)
    instance.following_count = randomize("int", min_val=1, max_val=1000)
    instance.id_ = randomize()
    instance.namespace = randomize("slug")
    instance.parent_namespace = randomize("slug")
    instance.total_liked_content = randomize("int", min_val=1, max_val=1000)
    return instance


def create_models_creator_response_example() -> ModelsCreatorResponse:
    instance = ModelsCreatorResponse()
    instance.creator_follow_state = create_models_creator_follow_state_example()
    instance.follow_count = randomize("int", min_val=1, max_val=1000)
    instance.following_count = randomize("int", min_val=1, max_val=1000)
    instance.id_ = randomize()
    instance.namespace = randomize("slug")
    instance.parent_namespace = randomize("slug")
    instance.total_liked_content = randomize("int", min_val=1, max_val=1000)
    return instance


def create_models_generate_content_upload_url_request_example() -> (
    ModelsGenerateContentUploadURLRequest
):
    instance = ModelsGenerateContentUploadURLRequest()
    instance.content_type = randomize()
    instance.file_extension = randomize()
    return instance


def create_models_generate_content_upload_url_response_example() -> (
    ModelsGenerateContentUploadURLResponse
):
    instance = ModelsGenerateContentUploadURLResponse()
    instance.file_location = randomize()
    instance.url = randomize("url")
    instance.content_type = randomize()
    instance.file_extension = randomize()
    return instance


def create_models_get_content_bulk_by_share_codes_request_example() -> (
    ModelsGetContentBulkByShareCodesRequest
):
    instance = ModelsGetContentBulkByShareCodesRequest()
    instance.share_codes = [randomize()]
    return instance


def create_models_get_content_preview_response_example() -> (
    ModelsGetContentPreviewResponse
):
    instance = ModelsGetContentPreviewResponse()
    instance.preview = randomize()
    instance.preview_url = randomize("url")
    return instance


def create_models_hide_content_request_example() -> ModelsHideContentRequest:
    instance = ModelsHideContentRequest()
    instance.is_hidden = randomize("bool")
    return instance


def create_models_like_state_example() -> ModelsLikeState:
    instance = ModelsLikeState()
    instance.state = randomize("bool")
    instance.user_id = randomize("uid")
    return instance


def create_models_list_content_versions_response_example() -> (
    ModelsListContentVersionsResponse
):
    instance = ModelsListContentVersionsResponse()
    instance.data = [create_models_content_version_response_example()]
    return instance


def create_models_paginated_content_download_response_example() -> (
    ModelsPaginatedContentDownloadResponse
):
    instance = ModelsPaginatedContentDownloadResponse()
    instance.data = [create_models_content_download_response_example()]
    instance.paging = create_models_paging_cursor_example()
    return instance


def create_models_paginated_content_download_response_v2_example() -> (
    ModelsPaginatedContentDownloadResponseV2
):
    instance = ModelsPaginatedContentDownloadResponseV2()
    instance.data = [create_models_content_download_response_v2_example()]
    instance.paging = create_models_paging_cursor_example()
    return instance


def create_models_paginated_content_downloader_response_example() -> (
    ModelsPaginatedContentDownloaderResponse
):
    instance = ModelsPaginatedContentDownloaderResponse()
    instance.data = [create_models_content_downloader_response_example()]
    instance.paging = create_models_paging_cursor_example()
    return instance


def create_models_paginated_content_likers_response_example() -> (
    ModelsPaginatedContentLikersResponse
):
    instance = ModelsPaginatedContentLikersResponse()
    instance.data = [create_models_content_likers_response_example()]
    instance.paging = create_models_paging_cursor_example()
    return instance


def create_models_paginated_creator_overview_response_example() -> (
    ModelsPaginatedCreatorOverviewResponse
):
    instance = ModelsPaginatedCreatorOverviewResponse()
    instance.data = [create_models_creator_overview_response_example()]
    instance.paging = create_models_paging_cursor_example()
    return instance


def create_models_paginated_get_channel_response_example() -> (
    ModelsPaginatedGetChannelResponse
):
    instance = ModelsPaginatedGetChannelResponse()
    instance.data = [create_models_channel_response_example()]
    instance.paging = create_models_paging_cursor_example()
    return instance


def create_models_paginated_get_configs_response_example() -> (
    ModelsPaginatedGetConfigsResponse
):
    instance = ModelsPaginatedGetConfigsResponse()
    instance.data = [create_models_config_response_example()]
    instance.paging = create_models_paging_cursor_example()
    return instance


def create_models_paginated_get_tag_response_example() -> ModelsPaginatedGetTagResponse:
    instance = ModelsPaginatedGetTagResponse()
    instance.data = [create_models_create_tag_response_example()]
    instance.paging = create_models_paging_cursor_example()
    return instance


def create_models_paginated_get_type_response_example() -> (
    ModelsPaginatedGetTypeResponse
):
    instance = ModelsPaginatedGetTypeResponse()
    instance.data = [create_models_create_type_response_example()]
    instance.paging = create_models_paging_cursor_example()
    return instance


def create_models_paginated_group_response_example() -> ModelsPaginatedGroupResponse:
    instance = ModelsPaginatedGroupResponse()
    instance.data = [create_models_create_group_response_example()]
    instance.paging = create_models_paging_cursor_example()
    return instance


def create_models_paginated_list_staging_content_response_example() -> (
    ModelsPaginatedListStagingContentResponse
):
    instance = ModelsPaginatedListStagingContentResponse()
    instance.data = [create_models_staging_content_response_example()]
    instance.paging = create_models_paging_cursor_example()
    return instance


def create_models_paging_cursor_example() -> ModelsPagingCursor:
    instance = ModelsPagingCursor()
    instance.first = randomize()
    instance.last = randomize()
    instance.next_ = randomize()
    instance.previous = randomize()
    return instance


def create_models_payload_url_example() -> ModelsPayloadURL:
    instance = ModelsPayloadURL()
    instance.source = randomize()
    instance.url = randomize("url")
    return instance


def create_models_preview_metadata_example() -> ModelsPreviewMetadata:
    instance = ModelsPreviewMetadata()
    instance.preview_content_type = randomize()
    instance.preview_file_extension = randomize()
    return instance


def create_models_preview_url_example() -> ModelsPreviewURL:
    instance = ModelsPreviewURL()
    instance.source = randomize()
    instance.url = randomize("url")
    return instance


def create_models_public_channel_request_example() -> ModelsPublicChannelRequest:
    instance = ModelsPublicChannelRequest()
    instance.name = randomize()
    return instance


def create_models_public_create_content_request_s3_example() -> (
    ModelsPublicCreateContentRequestS3
):
    instance = ModelsPublicCreateContentRequestS3()
    instance.name = randomize()
    instance.preview = randomize()
    instance.sub_type = randomize()
    instance.tags = [randomize()]
    instance.type_ = randomize()
    instance.content_type = randomize()
    instance.custom_attributes = {randomize(): randomize()}
    instance.file_extension = randomize()
    instance.preview_metadata = create_models_preview_metadata_example()
    return instance


def create_models_public_get_content_bulk_request_example() -> (
    ModelsPublicGetContentBulkRequest
):
    instance = ModelsPublicGetContentBulkRequest()
    instance.content_ids = [randomize()]
    return instance


def create_models_screenshot_response_example() -> ModelsScreenshotResponse:
    instance = ModelsScreenshotResponse()
    instance.description = randomize()
    instance.file_extension = randomize()
    instance.screenshot_id = randomize()
    instance.source = randomize()
    instance.url = randomize("url")
    instance.content_type = randomize()
    return instance


def create_models_staging_content_response_example() -> ModelsStagingContentResponse:
    instance = ModelsStagingContentResponse()
    instance.channel_id = randomize()
    instance.created_time = randomize("date")
    instance.id_ = randomize()
    instance.name = randomize()
    instance.namespace = randomize("slug")
    instance.status = randomize()
    instance.tags = [randomize()]
    instance.user_id = randomize("uid")
    instance.custom_attributes = {randomize(): randomize()}
    instance.file_extension = randomize()
    instance.note = randomize()
    instance.payload_url = randomize("url")
    instance.screenshots = [create_models_screenshot_response_example()]
    instance.sub_type = randomize()
    instance.type_ = randomize()
    instance.updated_time = randomize("date")
    return instance


def create_models_update_channel_request_example() -> ModelsUpdateChannelRequest:
    instance = ModelsUpdateChannelRequest()
    instance.name = randomize()
    return instance


def create_models_update_config_request_example() -> ModelsUpdateConfigRequest:
    instance = ModelsUpdateConfigRequest()
    instance.value = randomize()
    return instance


def create_models_update_content_request_example() -> ModelsUpdateContentRequest:
    instance = ModelsUpdateContentRequest()
    instance.content_type = randomize()
    instance.file_extension = randomize()
    instance.name = randomize()
    instance.payload = randomize()
    instance.preview = randomize()
    instance.sub_type = randomize()
    instance.tags = [randomize()]
    instance.type_ = randomize()
    instance.update_content_file = randomize("bool")
    instance.custom_attributes = {randomize(): randomize()}
    instance.preview_metadata = create_models_preview_metadata_example()
    return instance


def create_models_update_content_request_v2_example() -> ModelsUpdateContentRequestV2:
    instance = ModelsUpdateContentRequestV2()
    instance.custom_attributes = {randomize(): randomize()}
    instance.name = randomize()
    instance.sub_type = randomize()
    instance.tags = [randomize()]
    instance.type_ = randomize()
    return instance


def create_models_update_content_response_v2_example() -> ModelsUpdateContentResponseV2:
    instance = ModelsUpdateContentResponseV2()
    instance.channel_id = randomize()
    instance.content_status = randomize()
    instance.created_time = randomize()
    instance.file_location = randomize()
    instance.id_ = randomize()
    instance.is_hidden = randomize("bool")
    instance.is_official = randomize("bool")
    instance.name = randomize()
    instance.namespace = randomize("slug")
    instance.parent_namespace = randomize("slug")
    instance.share_code = randomize()
    instance.tags = [randomize()]
    instance.user_id = randomize("uid")
    instance.content_type = randomize()
    instance.custom_attributes = {randomize(): randomize()}
    instance.file_extension = randomize()
    instance.payload_url = randomize("url")
    instance.sub_type = randomize()
    instance.type_ = randomize()
    instance.updated_time = randomize()
    return instance


def create_models_update_content_share_code_request_example() -> (
    ModelsUpdateContentShareCodeRequest
):
    instance = ModelsUpdateContentShareCodeRequest()
    instance.share_code = randomize()
    return instance


def create_models_update_file_location_request_example() -> (
    ModelsUpdateFileLocationRequest
):
    instance = ModelsUpdateFileLocationRequest()
    instance.file_location = randomize()
    instance.file_extension = randomize()
    return instance


def create_models_update_screenshot_example() -> ModelsUpdateScreenshot:
    instance = ModelsUpdateScreenshot()
    instance.description = randomize()
    instance.screenshot_id = randomize()
    return instance


def create_models_update_screenshot_request_example() -> ModelsUpdateScreenshotRequest:
    instance = ModelsUpdateScreenshotRequest()
    instance.screenshots = [create_models_update_screenshot_example()]
    return instance


def create_models_update_screenshot_response_example() -> (
    ModelsUpdateScreenshotResponse
):
    instance = ModelsUpdateScreenshotResponse()
    instance.screenshots = [create_models_update_screenshot_example()]
    return instance


def create_models_update_staging_content_request_example() -> (
    ModelsUpdateStagingContentRequest
):
    instance = ModelsUpdateStagingContentRequest()
    instance.file_location = randomize()
    instance.file_extension = randomize()
    return instance


def create_models_user_follow_request_example() -> ModelsUserFollowRequest:
    instance = ModelsUserFollowRequest()
    instance.follow_status = randomize("bool")
    return instance


def create_models_user_follow_response_example() -> ModelsUserFollowResponse:
    instance = ModelsUserFollowResponse()
    instance.follow_status = randomize("bool")
    instance.user_id = randomize("uid")
    return instance


def create_response_error_example() -> ResponseError:
    instance = ResponseError()
    instance.error_code = randomize("int", min_val=1, max_val=1000)
    instance.error_message = randomize()
    return instance



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

# AccelByte Gaming Services Statistics Service

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from .....core import Operation
from .....core import HeaderStr
from .....core import HttpResponse

from ...models import ErrorEntity
from ...models import StatItemIncResult
from ...models import StatItemUpdate
from ...models import ValidationErrorEntity


class UpdateUserStatItemValue(Operation):
    """Update user's statitem value (updateUserStatItemValue)

    Update user's statitem value for a given namespace and user with a certain update strategy.
    There are four supported update strategies:
    + *OVERRIDE*: update user statitem with the new value
    + *INCREMENT*: increment user statitem with the specified value
    + *MAX*: update user statitem with the specified value if it's larger than the existing value
    + *MIN*: update user statitem with the specified value if it's lower than the existing value

    The *additionalKey* parameter will be suffixed to *userId* and is used to support multi level user's statitems, such as character's statitems.
    If provided, user's statitems will be saved with key: *userId_additionalKey*

    Other detail info:
    + *Returns*: updated user's statItem

    Properties:
        url: /social/v2/admin/namespaces/{namespace}/users/{userId}/stats/{statCode}/statitems/value

        method: PUT

        tags: ["UserStatistic"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) OPTIONAL StatItemUpdate in body

        namespace: (namespace) REQUIRED str in path

        stat_code: (statCode) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

        additional_key: (additionalKey) OPTIONAL str in query

    Responses:
        200: OK - StatItemIncResult (successful operation)

        400: Bad Request - ErrorEntity (12221: Invalid stat operator, expect [{expected}] but actual [{actual}])

        401: Unauthorized - ErrorEntity (20001: unauthorized access)

        403: Forbidden - ErrorEntity (20013: insufficient permission)

        404: Not Found - ErrorEntity (12241: Stat [{statCode}] cannot be found in namespace [{namespace}])

        409: Conflict - ErrorEntity (12273: Stat [{statCode}] is not decreasable | 12275: [{action}] value: [{value}] of stat [{statCode}] is out of range while minimum [{minimum}] and maximum [{maximum}] in namespace [{namespace}])

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)

        500: Internal Server Error - ErrorEntity (20000: Internal server error)
    """

    # region fields

    _url: str = "/social/v2/admin/namespaces/{namespace}/users/{userId}/stats/{statCode}/statitems/value"
    _method: str = "PUT"
    _consumes: List[str] = ["application/json"]
    _produces: List[str] = ["application/json"]
    _securities: List[List[str]] = [["BEARER_AUTH"]]
    _location_query: str = None

    body: StatItemUpdate  # OPTIONAL in [body]
    namespace: str  # REQUIRED in [path]
    stat_code: str  # REQUIRED in [path]
    user_id: str  # REQUIRED in [path]
    additional_key: str  # OPTIONAL in [query]

    # endregion fields

    # region properties

    @property
    def url(self) -> str:
        return self._url

    @property
    def method(self) -> str:
        return self._method

    @property
    def consumes(self) -> List[str]:
        return self._consumes

    @property
    def produces(self) -> List[str]:
        return self._produces

    @property
    def securities(self) -> List[List[str]]:
        return self._securities

    @property
    def location_query(self) -> str:
        return self._location_query

    # endregion properties

    # region get methods

    # endregion get methods

    # region get_x_params methods

    def get_all_params(self) -> dict:
        return {
            "body": self.get_body_params(),
            "path": self.get_path_params(),
            "query": self.get_query_params(),
        }

    def get_body_params(self) -> Any:
        if not hasattr(self, "body") or self.body is None:
            return None
        return self.body.to_dict()

    def get_path_params(self) -> dict:
        result = {}
        if hasattr(self, "namespace"):
            result["namespace"] = self.namespace
        if hasattr(self, "stat_code"):
            result["statCode"] = self.stat_code
        if hasattr(self, "user_id"):
            result["userId"] = self.user_id
        return result

    def get_query_params(self) -> dict:
        result = {}
        if hasattr(self, "additional_key"):
            result["additionalKey"] = self.additional_key
        return result

    # endregion get_x_params methods

    # region is/has methods

    # endregion is/has methods

    # region with_x methods

    def with_body(self, value: StatItemUpdate) -> UpdateUserStatItemValue:
        self.body = value
        return self

    def with_namespace(self, value: str) -> UpdateUserStatItemValue:
        self.namespace = value
        return self

    def with_stat_code(self, value: str) -> UpdateUserStatItemValue:
        self.stat_code = value
        return self

    def with_user_id(self, value: str) -> UpdateUserStatItemValue:
        self.user_id = value
        return self

    def with_additional_key(self, value: str) -> UpdateUserStatItemValue:
        self.additional_key = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "body") and self.body:
            result["body"] = self.body.to_dict(include_empty=include_empty)
        elif include_empty:
            result["body"] = StatItemUpdate()
        if hasattr(self, "namespace") and self.namespace:
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "stat_code") and self.stat_code:
            result["statCode"] = str(self.stat_code)
        elif include_empty:
            result["statCode"] = ""
        if hasattr(self, "user_id") and self.user_id:
            result["userId"] = str(self.user_id)
        elif include_empty:
            result["userId"] = ""
        if hasattr(self, "additional_key") and self.additional_key:
            result["additionalKey"] = str(self.additional_key)
        elif include_empty:
            result["additionalKey"] = ""
        return result

    # endregion to methods

    # region response methods

    # noinspection PyMethodMayBeStatic
    def parse_response(
        self, code: int, content_type: str, content: Any
    ) -> Tuple[
        Union[None, StatItemIncResult],
        Union[None, ErrorEntity, HttpResponse, ValidationErrorEntity],
    ]:
        """Parse the given response.

        200: OK - StatItemIncResult (successful operation)

        400: Bad Request - ErrorEntity (12221: Invalid stat operator, expect [{expected}] but actual [{actual}])

        401: Unauthorized - ErrorEntity (20001: unauthorized access)

        403: Forbidden - ErrorEntity (20013: insufficient permission)

        404: Not Found - ErrorEntity (12241: Stat [{statCode}] cannot be found in namespace [{namespace}])

        409: Conflict - ErrorEntity (12273: Stat [{statCode}] is not decreasable | 12275: [{action}] value: [{value}] of stat [{statCode}] is out of range while minimum [{minimum}] and maximum [{maximum}] in namespace [{namespace}])

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)

        500: Internal Server Error - ErrorEntity (20000: Internal server error)

        ---: HttpResponse (Undocumented Response)

        ---: HttpResponse (Unexpected Content-Type Error)

        ---: HttpResponse (Unhandled Error)
        """
        pre_processed_response, error = self.pre_process_response(
            code=code, content_type=content_type, content=content
        )
        if error is not None:
            return None, None if error.is_no_content() else error
        code, content_type, content = pre_processed_response

        if code == 200:
            return StatItemIncResult.create_from_dict(content), None
        if code == 400:
            return None, ErrorEntity.create_from_dict(content)
        if code == 401:
            return None, ErrorEntity.create_from_dict(content)
        if code == 403:
            return None, ErrorEntity.create_from_dict(content)
        if code == 404:
            return None, ErrorEntity.create_from_dict(content)
        if code == 409:
            return None, ErrorEntity.create_from_dict(content)
        if code == 422:
            return None, ValidationErrorEntity.create_from_dict(content)
        if code == 500:
            return None, ErrorEntity.create_from_dict(content)

        return self.handle_undocumented_response(
            code=code, content_type=content_type, content=content
        )

    # endregion response methods

    # region static methods

    @classmethod
    def create(
        cls,
        namespace: str,
        stat_code: str,
        user_id: str,
        body: Optional[StatItemUpdate] = None,
        additional_key: Optional[str] = None,
        **kwargs,
    ) -> UpdateUserStatItemValue:
        instance = cls()
        instance.namespace = namespace
        instance.stat_code = stat_code
        instance.user_id = user_id
        if body is not None:
            instance.body = body
        if additional_key is not None:
            instance.additional_key = additional_key
        if x_flight_id := kwargs.get("x_flight_id", None):
            instance.x_flight_id = x_flight_id
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> UpdateUserStatItemValue:
        instance = cls()
        if "body" in dict_ and dict_["body"] is not None:
            instance.body = StatItemUpdate.create_from_dict(
                dict_["body"], include_empty=include_empty
            )
        elif include_empty:
            instance.body = StatItemUpdate()
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "statCode" in dict_ and dict_["statCode"] is not None:
            instance.stat_code = str(dict_["statCode"])
        elif include_empty:
            instance.stat_code = ""
        if "userId" in dict_ and dict_["userId"] is not None:
            instance.user_id = str(dict_["userId"])
        elif include_empty:
            instance.user_id = ""
        if "additionalKey" in dict_ and dict_["additionalKey"] is not None:
            instance.additional_key = str(dict_["additionalKey"])
        elif include_empty:
            instance.additional_key = ""
        return instance

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "body": "body",
            "namespace": "namespace",
            "statCode": "stat_code",
            "userId": "user_id",
            "additionalKey": "additional_key",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "body": False,
            "namespace": True,
            "statCode": True,
            "userId": True,
            "additionalKey": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Challenge Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from .utils import randomize

from ..api.challenge.models import IamErrorResponse
from ..api.challenge.models import IamPermission
from ..api.challenge.models import ModelChallengeResponse
from ..api.challenge.models import ModelClaimUserRewardsReq
from ..api.challenge.models import ModelClaimUsersRewardsRequest
from ..api.challenge.models import ModelClaimUsersRewardsResponse
from ..api.challenge.models import ModelCreateChallengeRequest
from ..api.challenge.models import ModelCreateGoalRequest
from ..api.challenge.models import ModelEvaluatePlayerProgressionRequest
from ..api.challenge.models import ModelGetGoalsResponse
from ..api.challenge.models import ModelGoalMeta
from ..api.challenge.models import ModelGoalOrder
from ..api.challenge.models import ModelGoalProgressionResponse
from ..api.challenge.models import ModelGoalResponse
from ..api.challenge.models import ModelGoalSchedule
from ..api.challenge.models import ModelListChallengeResponse
from ..api.challenge.models import ModelListPeriodsResponse
from ..api.challenge.models import ModelListUserRewardsResponse
from ..api.challenge.models import ModelPagination
from ..api.challenge.models import ModelPluginAsignmentAppConfig
from ..api.challenge.models import ModelPluginAssignmentCustomConfig
from ..api.challenge.models import ModelPluginAssignmentRequest
from ..api.challenge.models import ModelPluginAssignmentResponse
from ..api.challenge.models import ModelPredicate
from ..api.challenge.models import ModelRequirement
from ..api.challenge.models import ModelRequirementProgressionResponse
from ..api.challenge.models import ModelResetConfig
from ..api.challenge.models import ModelReward
from ..api.challenge.models import ModelSchedule
from ..api.challenge.models import ModelUpdateChallengeRequest
from ..api.challenge.models import ModelUpdateGoalRequest
from ..api.challenge.models import ModelUserProgressionPeriodResponseMeta
from ..api.challenge.models import ModelUserProgressionResponse
from ..api.challenge.models import ModelUserProgressionResponseMeta
from ..api.challenge.models import ModelUserReward
from ..api.challenge.models import ModelsPeriod
from ..api.challenge.models import ResponseError


def create_iam_error_response_example() -> IamErrorResponse:
    instance = IamErrorResponse()
    instance.error_code = randomize("int", min_val=1, max_val=1000)
    instance.error_message = randomize()
    instance.required_permission = create_iam_permission_example()
    return instance


def create_iam_permission_example() -> IamPermission:
    instance = IamPermission()
    instance.action = randomize("int", min_val=1, max_val=1000)
    instance.resource = randomize()
    return instance


def create_model_challenge_response_example() -> ModelChallengeResponse:
    instance = ModelChallengeResponse()
    instance.active_goals_per_rotation = randomize("int", min_val=1, max_val=1000)
    instance.assignment_rule = randomize()
    instance.code = randomize()
    instance.created_at = randomize()
    instance.description = randomize()
    instance.goals_visibility = randomize()
    instance.name = randomize()
    instance.randomized_per_rotation = randomize("bool")
    instance.reset_config = create_model_reset_config_example()
    instance.rotation = randomize()
    instance.start_date = randomize()
    instance.status = randomize()
    instance.updated_at = randomize()
    instance.end_after = randomize("int", min_val=1, max_val=1000)
    instance.end_date = randomize()
    instance.repeat_after = randomize("int", min_val=1, max_val=1000)
    return instance


def create_model_claim_user_rewards_req_example() -> ModelClaimUserRewardsReq:
    instance = ModelClaimUserRewardsReq()
    instance.reward_i_ds = [randomize()]
    return instance


def create_model_claim_users_rewards_request_example() -> ModelClaimUsersRewardsRequest:
    instance = ModelClaimUsersRewardsRequest()
    instance.reward_ids = [randomize()]
    instance.user_id = randomize("uid")
    return instance


def create_model_claim_users_rewards_response_example() -> (
    ModelClaimUsersRewardsResponse
):
    instance = ModelClaimUsersRewardsResponse()
    instance.is_success = randomize("bool")
    instance.rewards = [create_model_user_reward_example()]
    instance.user_id = randomize("uid")
    instance.error_detail = create_response_error_example()
    return instance


def create_model_create_challenge_request_example() -> ModelCreateChallengeRequest:
    instance = ModelCreateChallengeRequest()
    instance.assignment_rule = randomize()
    instance.code = randomize()
    instance.goals_visibility = randomize()
    instance.name = randomize()
    instance.rotation = randomize()
    instance.start_date = randomize("date")
    instance.active_goals_per_rotation = randomize("int", min_val=1, max_val=1000)
    instance.description = randomize()
    instance.end_after = randomize("int", min_val=1, max_val=1000)
    instance.end_date = randomize("date")
    instance.randomized_per_rotation = randomize("bool")
    instance.repeat_after = randomize("int", min_val=1, max_val=1000)
    instance.reset_config = create_model_reset_config_example()
    return instance


def create_model_create_goal_request_example() -> ModelCreateGoalRequest:
    instance = ModelCreateGoalRequest()
    instance.code = randomize()
    instance.is_active = randomize("bool")
    instance.name = randomize()
    instance.description = randomize()
    instance.requirement_groups = [create_model_requirement_example()]
    instance.rewards = [create_model_reward_example()]
    instance.schedule = create_model_goal_schedule_example()
    instance.tags = [randomize()]
    return instance


def create_model_evaluate_player_progression_request_example() -> (
    ModelEvaluatePlayerProgressionRequest
):
    instance = ModelEvaluatePlayerProgressionRequest()
    instance.user_ids = [randomize()]
    return instance


def create_model_get_goals_response_example() -> ModelGetGoalsResponse:
    instance = ModelGetGoalsResponse()
    instance.data = [create_model_goal_response_example()]
    instance.meta = create_model_challenge_response_example()
    instance.paging = create_model_pagination_example()
    return instance


def create_model_goal_meta_example() -> ModelGoalMeta:
    instance = ModelGoalMeta()
    instance.code = randomize()
    instance.description = randomize()
    instance.name = randomize()
    instance.requirement_groups = [create_model_requirement_example()]
    instance.rewards = [create_model_reward_example()]
    instance.tags = [randomize()]
    return instance


def create_model_goal_order_example() -> ModelGoalOrder:
    instance = ModelGoalOrder()
    instance.goal = create_model_goal_response_example()
    instance.order = randomize("int", min_val=1, max_val=1000)
    return instance


def create_model_goal_progression_response_example() -> ModelGoalProgressionResponse:
    instance = ModelGoalProgressionResponse()
    instance.challenge_code = randomize()
    instance.goal = create_model_goal_meta_example()
    instance.goal_code = randomize()
    instance.goal_progression_id = randomize()
    instance.requirement_progressions = [
        create_model_requirement_progression_response_example()
    ]
    instance.status = randomize()
    return instance


def create_model_goal_response_example() -> ModelGoalResponse:
    instance = ModelGoalResponse()
    instance.challenge_code = randomize()
    instance.code = randomize()
    instance.created_at = randomize()
    instance.description = randomize()
    instance.is_active = randomize("bool")
    instance.name = randomize()
    instance.namespace = randomize("slug")
    instance.requirement_groups = [create_model_requirement_example()]
    instance.rewards = [create_model_reward_example()]
    instance.updated_at = randomize()
    instance.schedule = create_model_goal_schedule_example()
    instance.tags = [randomize()]
    return instance


def create_model_goal_schedule_example() -> ModelGoalSchedule:
    instance = ModelGoalSchedule()
    instance.order = randomize("int", min_val=1, max_val=1000)
    instance.start_time = randomize("date")
    instance.end_time = randomize("date")
    return instance


def create_model_list_challenge_response_example() -> ModelListChallengeResponse:
    instance = ModelListChallengeResponse()
    instance.data = [create_model_challenge_response_example()]
    instance.paging = create_model_pagination_example()
    return instance


def create_model_list_periods_response_example() -> ModelListPeriodsResponse:
    instance = ModelListPeriodsResponse()
    instance.data = [create_models_period_example()]
    instance.paging = create_model_pagination_example()
    return instance


def create_model_list_user_rewards_response_example() -> ModelListUserRewardsResponse:
    instance = ModelListUserRewardsResponse()
    instance.data = [create_model_user_reward_example()]
    instance.paging = create_model_pagination_example()
    return instance


def create_model_pagination_example() -> ModelPagination:
    instance = ModelPagination()
    instance.first = randomize()
    instance.last = randomize()
    instance.next_ = randomize()
    instance.previous = randomize()
    return instance


def create_model_plugin_asignment_app_config_example() -> ModelPluginAsignmentAppConfig:
    instance = ModelPluginAsignmentAppConfig()
    instance.app_name = randomize()
    return instance


def create_model_plugin_assignment_custom_config_example() -> (
    ModelPluginAssignmentCustomConfig
):
    instance = ModelPluginAssignmentCustomConfig()
    instance.grpc_server_address = randomize()
    return instance


def create_model_plugin_assignment_request_example() -> ModelPluginAssignmentRequest:
    instance = ModelPluginAssignmentRequest()
    instance.extend_type = randomize()
    instance.app_name = randomize()
    instance.grpc_server_address = randomize()
    return instance


def create_model_plugin_assignment_response_example() -> ModelPluginAssignmentResponse:
    instance = ModelPluginAssignmentResponse()
    instance.extend_type = randomize()
    instance.app_config = create_model_plugin_asignment_app_config_example()
    instance.custom_config = create_model_plugin_assignment_custom_config_example()
    return instance


def create_model_predicate_example() -> ModelPredicate:
    instance = ModelPredicate()
    instance.matcher = randomize()
    instance.parameter_name = randomize()
    instance.parameter_type = randomize()
    instance.target_value = randomize("int", min_val=1, max_val=1000)
    instance.stat_cycle_id = randomize()
    return instance


def create_model_requirement_example() -> ModelRequirement:
    instance = ModelRequirement()
    instance.operator = randomize()
    instance.predicates = [create_model_predicate_example()]
    return instance


def create_model_requirement_progression_response_example() -> (
    ModelRequirementProgressionResponse
):
    instance = ModelRequirementProgressionResponse()
    instance.current_value = randomize("int", min_val=1, max_val=1000)
    instance.id_ = randomize()
    instance.matcher = randomize()
    instance.parameter_name = randomize()
    instance.parameter_type = randomize()
    instance.target_value = randomize("int", min_val=1, max_val=1000)
    instance.completed_at = randomize()
    instance.stat_cycle_id = randomize()
    return instance


def create_model_reset_config_example() -> ModelResetConfig:
    instance = ModelResetConfig()
    instance.reset_date = randomize("int", min_val=1, max_val=1000)
    instance.reset_day = randomize("int", min_val=1, max_val=1000)
    instance.reset_time = randomize()
    return instance


def create_model_reward_example() -> ModelReward:
    instance = ModelReward()
    instance.item_id = randomize()
    instance.item_name = randomize()
    instance.qty = randomize("int", min_val=1, max_val=1000)
    instance.type_ = randomize()
    return instance


def create_model_schedule_example() -> ModelSchedule:
    instance = ModelSchedule()
    instance.challenge_code = randomize()
    instance.created_at = randomize()
    instance.end_time = randomize()
    instance.goals = [create_model_goal_order_example()]
    instance.namespace = randomize("slug")
    instance.start_time = randomize()
    instance.updated_at = randomize()
    return instance


def create_model_update_challenge_request_example() -> ModelUpdateChallengeRequest:
    instance = ModelUpdateChallengeRequest()
    instance.active_goals_per_rotation = randomize("int", min_val=1, max_val=1000)
    instance.assignment_rule = randomize()
    instance.description = randomize()
    instance.end_after = randomize("int", min_val=1, max_val=1000)
    instance.end_date = randomize("date")
    instance.goals_visibility = randomize()
    instance.name = randomize()
    instance.randomized_per_rotation = randomize("bool")
    instance.repeat_after = randomize("int", min_val=1, max_val=1000)
    instance.reset_config = create_model_reset_config_example()
    instance.rotation = randomize()
    instance.start_date = randomize("date")
    return instance


def create_model_update_goal_request_example() -> ModelUpdateGoalRequest:
    instance = ModelUpdateGoalRequest()
    instance.name = randomize()
    instance.description = randomize()
    instance.is_active = randomize("bool")
    instance.requirement_groups = [create_model_requirement_example()]
    instance.rewards = [create_model_reward_example()]
    instance.schedule = create_model_goal_schedule_example()
    instance.tags = [randomize()]
    return instance


def create_model_user_progression_period_response_meta_example() -> (
    ModelUserProgressionPeriodResponseMeta
):
    instance = ModelUserProgressionPeriodResponseMeta()
    instance.end_time = randomize()
    instance.start_time = randomize()
    return instance


def create_model_user_progression_response_example() -> ModelUserProgressionResponse:
    instance = ModelUserProgressionResponse()
    instance.data = [create_model_goal_progression_response_example()]
    instance.meta = create_model_user_progression_response_meta_example()
    instance.paging = create_model_pagination_example()
    return instance


def create_model_user_progression_response_meta_example() -> (
    ModelUserProgressionResponseMeta
):
    instance = ModelUserProgressionResponseMeta()
    instance.code = randomize()
    instance.description = randomize()
    instance.name = randomize()
    instance.period = create_model_user_progression_period_response_meta_example()
    instance.start_date = randomize()
    instance.user_id = randomize("uid")
    instance.end_date = randomize()
    return instance


def create_model_user_reward_example() -> ModelUserReward:
    instance = ModelUserReward()
    instance.challenge_code = randomize()
    instance.created_at = randomize()
    instance.goal_code = randomize()
    instance.id_ = randomize()
    instance.item_id = randomize()
    instance.item_name = randomize()
    instance.qty = randomize("int", min_val=1, max_val=1000)
    instance.status = randomize()
    instance.type_ = randomize()
    instance.updated_at = randomize()
    instance.user_id = randomize("uid")
    return instance


def create_models_period_example() -> ModelsPeriod:
    instance = ModelsPeriod()
    instance.end_time = randomize("date")
    instance.slot = randomize("int", min_val=1, max_val=1000)
    instance.start_time = randomize("date")
    return instance


def create_response_error_example() -> ResponseError:
    instance = ResponseError()
    instance.attributes = {randomize(): randomize()}
    instance.error_code = randomize("int", min_val=1, max_val=1000)
    instance.error_message = randomize()
    instance.message = randomize()
    instance.name = randomize()
    return instance



from typing import List, Optional

from ._cache_types import NamespaceContextCache
from ._ctypes import PermissionStruct


STUDIO_GAME_DELIMITER: str = "-"


def validate_action(action: int, expected_action: int) -> bool:
    return (action & expected_action) == expected_action


def validate_resource(
    resource: str,
    expected_resource: str,
    namespace_context_cache: Optional[NamespaceContextCache] = None,
) -> bool:
    # Split both the resource and the expected resource into tokens with ':' as delimiter.
    r_items = resource.split(":")
    e_items = expected_resource.split(":")

    r_items_len = len(r_items)
    e_items_len = len(e_items)
    min_len = min(r_items_len, e_items_len)

    # Compare each token in the resource and the expected resource from left to right.
    for i in range(min_len):
        r_token = r_items[i]
        e_token = e_items[i]
        # If token string match or other one is '*', then proceed to next token.
        # Otherwise, we can stop here because the resource does not match.
        # |                       vvv : DOES NOT MATCH
        # | resource(1):          BAR
        # | expected_resource(1): FOO
        if r_token != e_token and r_token != "*":
            if r_token.endswith("-") and i > 0:
                prev_r_token = r_items[i - 1]
                if prev_r_token == "NAMESPACE":
                    if (
                        STUDIO_GAME_DELIMITER in e_token
                        and len(e_token.split(STUDIO_GAME_DELIMITER)) == 2
                        and e_token.startswith(r_token)
                    ):
                        continue
                    if r_token == f"{e_token}{STUDIO_GAME_DELIMITER}":
                        continue
                    if namespace_context_cache:
                        context = namespace_context_cache.get_namespace_context(
                            namespace=e_token
                        )
                        if (
                            context is not None
                            and context.type_ == "Game"
                            and r_token.startswith(context.studio_namespace)
                        ):
                            continue
            return False

    # If the number of resource tokens < the number of expected resource tokens.
    if r_items_len < e_items_len:
        # If the last resource token is '*',
        # we can stop here because the resource does not match.
        # |                           vvv : DOES NOT MATCH!
        # | resource(2):          FOO:BAR
        # | expected_resource(3): FOO:BAR:BAZ
        if r_items[-1] != "*":
            return False

        # If the second resource token from the right is 'NAMESPACE' or 'USER',
        # we can stop here because the resource does not match.
        # |                           vvvv : DOES NOT MATCH
        # | resource(3):          FOO:USER:*
        # | expected_resource(4): FOO:BAR:BAZ:QUX
        elif r_items_len >= 2 and r_items[-2] in ("NAMESPACE", "USER"):
            return False

    # If the number resource tokens > the number of expected resource tokens.
    if r_items_len > e_items_len:
        for i in range(e_items_len, r_items_len):
            # If the remaining resource tokens are ALL '*', then proceed.
            # Otherwise, we can stop here and the resource does not match.
            # |                           vvv : DOES NOT MATCH!
            # | resource(2):          FOO:BAR
            # | expected_resource(1): FOO
            if r_items[i] != "*":
                return False

    return True


def validate_permission(
    target: PermissionStruct,
    permissions: List[PermissionStruct],
    namespace_context_cache: Optional[NamespaceContextCache] = None,
) -> bool:
    for permission in permissions:
        if validate_action(permission.action, target.action) and validate_resource(
            permission.resource, target.resource, namespace_context_cache
        ):
            return True

    return False



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

# AccelByte Gaming Services Session Service

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from .....core import Operation
from .....core import HeaderStr
from .....core import HttpResponse

from ...models import ApimodelsGameSessionQueryResponse
from ...models import ResponseError


class PublicQueryGameSessionsByAttributes(Operation):
    """Query game sessions (publicQueryGameSessionsByAttributes)

    Query game sessions.

    By default, API will return a list of available game sessions (joinability: open).
    Session service has several DSInformation status to track DS request to DS providers:
    - NEED_TO_REQUEST: number of active players hasn't reached session's minPlayers therefore DS has not yet requested.
    - REQUESTED: DS is being requested to DSMC.
    - PREPARING: DS needs to call manual set ready for the game session
    - AVAILABLE: DS is ready to use. The DSMC status for this DS is either READY/BUSY.
    - FAILED_TO_REQUEST: DSMC fails to spin up a DS for session.
    - DS_ERROR: DS provider fails to spin up the DS or the DS itself becomes unreachable
    - DS_CANCELLED: when DSMC is preparing the DS, DSMC will give a temporary DS. In this phase, if you delete the game session, the DS request will be canceled.
    - ENDED: when a game session (match) has finished and DS has done its job, it will terminate itself.
    - UNKNOWN: if any unknown DS status is detected.
    DSInformation has 2 fields for DS status: "status" and "statusV2". The "status" is there for backward-compatibility, therefore we encourage to just rely on "statusV2" for the more updated statuses.
    DS Source can be DSMC, AMS or custom. In DSMC, a DS request will be put in queue if they dont have available buffers, and DSMC will let the service knows when they finished spinning it up. While AMS doesn't have a concept of queue. Therefore some "DSInformation.statusV2" only applicable for DSMC.

    query parameter "availability" to filter sessions' availability:
    all: return all sessions regardless it's full
    full: only return active sessions
    default behavior (unset or else): return only available sessions (not full)

    Properties:
        url: /session/v1/public/namespaces/{namespace}/gamesessions

        method: POST

        tags: ["Game Session"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED Dict[str, Any] in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApimodelsGameSessionQueryResponse (OK)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """

    # region fields

    _url: str = "/session/v1/public/namespaces/{namespace}/gamesessions"
    _method: str = "POST"
    _consumes: List[str] = ["application/json"]
    _produces: List[str] = ["application/json"]
    _securities: List[List[str]] = [["BEARER_AUTH"]]
    _location_query: str = None

    body: Dict[str, Any]  # REQUIRED in [body]
    namespace: str  # REQUIRED in [path]

    # endregion fields

    # region properties

    @property
    def url(self) -> str:
        return self._url

    @property
    def method(self) -> str:
        return self._method

    @property
    def consumes(self) -> List[str]:
        return self._consumes

    @property
    def produces(self) -> List[str]:
        return self._produces

    @property
    def securities(self) -> List[List[str]]:
        return self._securities

    @property
    def location_query(self) -> str:
        return self._location_query

    # endregion properties

    # region get methods

    # endregion get methods

    # region get_x_params methods

    def get_all_params(self) -> dict:
        return {
            "body": self.get_body_params(),
            "path": self.get_path_params(),
        }

    def get_body_params(self) -> Any:
        if not hasattr(self, "body") or self.body is None:
            return None
        return self.body

    def get_path_params(self) -> dict:
        result = {}
        if hasattr(self, "namespace"):
            result["namespace"] = self.namespace
        return result

    # endregion get_x_params methods

    # region is/has methods

    # endregion is/has methods

    # region with_x methods

    def with_body(self, value: Dict[str, Any]) -> PublicQueryGameSessionsByAttributes:
        self.body = value
        return self

    def with_namespace(self, value: str) -> PublicQueryGameSessionsByAttributes:
        self.namespace = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "body") and self.body:
            result["body"] = {str(k0): v0 for k0, v0 in self.body.items()}
        elif include_empty:
            result["body"] = {}
        if hasattr(self, "namespace") and self.namespace:
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        return result

    # endregion to methods

    # region response methods

    # noinspection PyMethodMayBeStatic
    def parse_response(
        self, code: int, content_type: str, content: Any
    ) -> Tuple[
        Union[None, ApimodelsGameSessionQueryResponse],
        Union[None, HttpResponse, ResponseError],
    ]:
        """Parse the given response.

        200: OK - ApimodelsGameSessionQueryResponse (OK)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        500: Internal Server Error - ResponseError (Internal Server Error)

        ---: HttpResponse (Undocumented Response)

        ---: HttpResponse (Unexpected Content-Type Error)

        ---: HttpResponse (Unhandled Error)
        """
        pre_processed_response, error = self.pre_process_response(
            code=code, content_type=content_type, content=content
        )
        if error is not None:
            return None, None if error.is_no_content() else error
        code, content_type, content = pre_processed_response

        if code == 200:
            return ApimodelsGameSessionQueryResponse.create_from_dict(content), None
        if code == 400:
            return None, ResponseError.create_from_dict(content)
        if code == 401:
            return None, ResponseError.create_from_dict(content)
        if code == 403:
            return None, ResponseError.create_from_dict(content)
        if code == 500:
            return None, ResponseError.create_from_dict(content)

        return self.handle_undocumented_response(
            code=code, content_type=content_type, content=content
        )

    # endregion response methods

    # region static methods

    @classmethod
    def create(
        cls, body: Dict[str, Any], namespace: str, **kwargs
    ) -> PublicQueryGameSessionsByAttributes:
        instance = cls()
        instance.body = body
        instance.namespace = namespace
        if x_flight_id := kwargs.get("x_flight_id", None):
            instance.x_flight_id = x_flight_id
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> PublicQueryGameSessionsByAttributes:
        instance = cls()
        if "body" in dict_ and dict_["body"] is not None:
            instance.body = {str(k0): v0 for k0, v0 in dict_["body"].items()}
        elif include_empty:
            instance.body = {}
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        return instance

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "body": "body",
            "namespace": "namespace",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "body": True,
            "namespace": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Chat Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class ModelsChannelTopicResponse(Model):
    """Models channel topic response (models.ChannelTopicResponse)

    Properties:
        created_at: (createdAt) REQUIRED int

        created_by: (createdBy) REQUIRED str

        description: (description) REQUIRED str

        is_joinable: (isJoinable) REQUIRED bool

        message_per_minutes: (messagePerMinutes) REQUIRED float

        name: (name) REQUIRED str

        namespace: (namespace) REQUIRED str

        shard_limit: (shardLimit) REQUIRED int

        shard_number: (shardNumber) REQUIRED int

        topic_id: (topicId) REQUIRED str

        total_member: (totalMember) REQUIRED int
    """

    # region fields

    created_at: int  # REQUIRED
    created_by: str  # REQUIRED
    description: str  # REQUIRED
    is_joinable: bool  # REQUIRED
    message_per_minutes: float  # REQUIRED
    name: str  # REQUIRED
    namespace: str  # REQUIRED
    shard_limit: int  # REQUIRED
    shard_number: int  # REQUIRED
    topic_id: str  # REQUIRED
    total_member: int  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_created_at(self, value: int) -> ModelsChannelTopicResponse:
        self.created_at = value
        return self

    def with_created_by(self, value: str) -> ModelsChannelTopicResponse:
        self.created_by = value
        return self

    def with_description(self, value: str) -> ModelsChannelTopicResponse:
        self.description = value
        return self

    def with_is_joinable(self, value: bool) -> ModelsChannelTopicResponse:
        self.is_joinable = value
        return self

    def with_message_per_minutes(self, value: float) -> ModelsChannelTopicResponse:
        self.message_per_minutes = value
        return self

    def with_name(self, value: str) -> ModelsChannelTopicResponse:
        self.name = value
        return self

    def with_namespace(self, value: str) -> ModelsChannelTopicResponse:
        self.namespace = value
        return self

    def with_shard_limit(self, value: int) -> ModelsChannelTopicResponse:
        self.shard_limit = value
        return self

    def with_shard_number(self, value: int) -> ModelsChannelTopicResponse:
        self.shard_number = value
        return self

    def with_topic_id(self, value: str) -> ModelsChannelTopicResponse:
        self.topic_id = value
        return self

    def with_total_member(self, value: int) -> ModelsChannelTopicResponse:
        self.total_member = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "created_at"):
            result["createdAt"] = int(self.created_at)
        elif include_empty:
            result["createdAt"] = 0
        if hasattr(self, "created_by"):
            result["createdBy"] = str(self.created_by)
        elif include_empty:
            result["createdBy"] = ""
        if hasattr(self, "description"):
            result["description"] = str(self.description)
        elif include_empty:
            result["description"] = ""
        if hasattr(self, "is_joinable"):
            result["isJoinable"] = bool(self.is_joinable)
        elif include_empty:
            result["isJoinable"] = False
        if hasattr(self, "message_per_minutes"):
            result["messagePerMinutes"] = float(self.message_per_minutes)
        elif include_empty:
            result["messagePerMinutes"] = 0.0
        if hasattr(self, "name"):
            result["name"] = str(self.name)
        elif include_empty:
            result["name"] = ""
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "shard_limit"):
            result["shardLimit"] = int(self.shard_limit)
        elif include_empty:
            result["shardLimit"] = 0
        if hasattr(self, "shard_number"):
            result["shardNumber"] = int(self.shard_number)
        elif include_empty:
            result["shardNumber"] = 0
        if hasattr(self, "topic_id"):
            result["topicId"] = str(self.topic_id)
        elif include_empty:
            result["topicId"] = ""
        if hasattr(self, "total_member"):
            result["totalMember"] = int(self.total_member)
        elif include_empty:
            result["totalMember"] = 0
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        created_at: int,
        created_by: str,
        description: str,
        is_joinable: bool,
        message_per_minutes: float,
        name: str,
        namespace: str,
        shard_limit: int,
        shard_number: int,
        topic_id: str,
        total_member: int,
        **kwargs,
    ) -> ModelsChannelTopicResponse:
        instance = cls()
        instance.created_at = created_at
        instance.created_by = created_by
        instance.description = description
        instance.is_joinable = is_joinable
        instance.message_per_minutes = message_per_minutes
        instance.name = name
        instance.namespace = namespace
        instance.shard_limit = shard_limit
        instance.shard_number = shard_number
        instance.topic_id = topic_id
        instance.total_member = total_member
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelsChannelTopicResponse:
        instance = cls()
        if not dict_:
            return instance
        if "createdAt" in dict_ and dict_["createdAt"] is not None:
            instance.created_at = int(dict_["createdAt"])
        elif include_empty:
            instance.created_at = 0
        if "createdBy" in dict_ and dict_["createdBy"] is not None:
            instance.created_by = str(dict_["createdBy"])
        elif include_empty:
            instance.created_by = ""
        if "description" in dict_ and dict_["description"] is not None:
            instance.description = str(dict_["description"])
        elif include_empty:
            instance.description = ""
        if "isJoinable" in dict_ and dict_["isJoinable"] is not None:
            instance.is_joinable = bool(dict_["isJoinable"])
        elif include_empty:
            instance.is_joinable = False
        if "messagePerMinutes" in dict_ and dict_["messagePerMinutes"] is not None:
            instance.message_per_minutes = float(dict_["messagePerMinutes"])
        elif include_empty:
            instance.message_per_minutes = 0.0
        if "name" in dict_ and dict_["name"] is not None:
            instance.name = str(dict_["name"])
        elif include_empty:
            instance.name = ""
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "shardLimit" in dict_ and dict_["shardLimit"] is not None:
            instance.shard_limit = int(dict_["shardLimit"])
        elif include_empty:
            instance.shard_limit = 0
        if "shardNumber" in dict_ and dict_["shardNumber"] is not None:
            instance.shard_number = int(dict_["shardNumber"])
        elif include_empty:
            instance.shard_number = 0
        if "topicId" in dict_ and dict_["topicId"] is not None:
            instance.topic_id = str(dict_["topicId"])
        elif include_empty:
            instance.topic_id = ""
        if "totalMember" in dict_ and dict_["totalMember"] is not None:
            instance.total_member = int(dict_["totalMember"])
        elif include_empty:
            instance.total_member = 0
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelsChannelTopicResponse]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelsChannelTopicResponse]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ModelsChannelTopicResponse,
        List[ModelsChannelTopicResponse],
        Dict[Any, ModelsChannelTopicResponse],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "createdAt": "created_at",
            "createdBy": "created_by",
            "description": "description",
            "isJoinable": "is_joinable",
            "messagePerMinutes": "message_per_minutes",
            "name": "name",
            "namespace": "namespace",
            "shardLimit": "shard_limit",
            "shardNumber": "shard_number",
            "topicId": "topic_id",
            "totalMember": "total_member",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "createdAt": True,
            "createdBy": True,
            "description": True,
            "isJoinable": True,
            "messagePerMinutes": True,
            "name": True,
            "namespace": True,
            "shardLimit": True,
            "shardNumber": True,
            "topicId": True,
            "totalMember": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Dsm Controller Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class ModelsDetailedCountServerResponse(Model):
    """Models detailed count server response (models.DetailedCountServerResponse)

    Properties:
        busy_count: (busy_count) REQUIRED int

        creating_count: (creating_count) REQUIRED int

        ready_count: (ready_count) REQUIRED int

        unreachable_count: (unreachable_count) REQUIRED int
    """

    # region fields

    busy_count: int  # REQUIRED
    creating_count: int  # REQUIRED
    ready_count: int  # REQUIRED
    unreachable_count: int  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_busy_count(self, value: int) -> ModelsDetailedCountServerResponse:
        self.busy_count = value
        return self

    def with_creating_count(self, value: int) -> ModelsDetailedCountServerResponse:
        self.creating_count = value
        return self

    def with_ready_count(self, value: int) -> ModelsDetailedCountServerResponse:
        self.ready_count = value
        return self

    def with_unreachable_count(self, value: int) -> ModelsDetailedCountServerResponse:
        self.unreachable_count = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "busy_count"):
            result["busy_count"] = int(self.busy_count)
        elif include_empty:
            result["busy_count"] = 0
        if hasattr(self, "creating_count"):
            result["creating_count"] = int(self.creating_count)
        elif include_empty:
            result["creating_count"] = 0
        if hasattr(self, "ready_count"):
            result["ready_count"] = int(self.ready_count)
        elif include_empty:
            result["ready_count"] = 0
        if hasattr(self, "unreachable_count"):
            result["unreachable_count"] = int(self.unreachable_count)
        elif include_empty:
            result["unreachable_count"] = 0
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        busy_count: int,
        creating_count: int,
        ready_count: int,
        unreachable_count: int,
        **kwargs,
    ) -> ModelsDetailedCountServerResponse:
        instance = cls()
        instance.busy_count = busy_count
        instance.creating_count = creating_count
        instance.ready_count = ready_count
        instance.unreachable_count = unreachable_count
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelsDetailedCountServerResponse:
        instance = cls()
        if not dict_:
            return instance
        if "busy_count" in dict_ and dict_["busy_count"] is not None:
            instance.busy_count = int(dict_["busy_count"])
        elif include_empty:
            instance.busy_count = 0
        if "creating_count" in dict_ and dict_["creating_count"] is not None:
            instance.creating_count = int(dict_["creating_count"])
        elif include_empty:
            instance.creating_count = 0
        if "ready_count" in dict_ and dict_["ready_count"] is not None:
            instance.ready_count = int(dict_["ready_count"])
        elif include_empty:
            instance.ready_count = 0
        if "unreachable_count" in dict_ and dict_["unreachable_count"] is not None:
            instance.unreachable_count = int(dict_["unreachable_count"])
        elif include_empty:
            instance.unreachable_count = 0
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelsDetailedCountServerResponse]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelsDetailedCountServerResponse]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ModelsDetailedCountServerResponse,
        List[ModelsDetailedCountServerResponse],
        Dict[Any, ModelsDetailedCountServerResponse],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "busy_count": "busy_count",
            "creating_count": "creating_count",
            "ready_count": "ready_count",
            "unreachable_count": "unreachable_count",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "busy_count": True,
            "creating_count": True,
            "ready_count": True,
            "unreachable_count": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Reporting Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model
from ....core import StrEnum


class CategoryEnum(StrEnum):
    CHAT = "CHAT"
    EXTENSION = "EXTENSION"
    UGC = "UGC"
    USER = "USER"


class RestapiSubmitReportRequest(Model):
    """Restapi submit report request (restapi.submitReportRequest)

    Properties:
        category: (category) REQUIRED Union[str, CategoryEnum]

        reason: (reason) REQUIRED str

        user_id: (userId) REQUIRED str

        additional_info: (additionalInfo) OPTIONAL Dict[str, Any]

        comment: (comment) OPTIONAL str

        extension_category: (extensionCategory) OPTIONAL str

        object_id: (objectId) OPTIONAL str

        object_type: (objectType) OPTIONAL str
    """

    # region fields

    category: Union[str, CategoryEnum]  # REQUIRED
    reason: str  # REQUIRED
    user_id: str  # REQUIRED
    additional_info: Dict[str, Any]  # OPTIONAL
    comment: str  # OPTIONAL
    extension_category: str  # OPTIONAL
    object_id: str  # OPTIONAL
    object_type: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_category(
        self, value: Union[str, CategoryEnum]
    ) -> RestapiSubmitReportRequest:
        self.category = value
        return self

    def with_reason(self, value: str) -> RestapiSubmitReportRequest:
        self.reason = value
        return self

    def with_user_id(self, value: str) -> RestapiSubmitReportRequest:
        self.user_id = value
        return self

    def with_additional_info(self, value: Dict[str, Any]) -> RestapiSubmitReportRequest:
        self.additional_info = value
        return self

    def with_comment(self, value: str) -> RestapiSubmitReportRequest:
        self.comment = value
        return self

    def with_extension_category(self, value: str) -> RestapiSubmitReportRequest:
        self.extension_category = value
        return self

    def with_object_id(self, value: str) -> RestapiSubmitReportRequest:
        self.object_id = value
        return self

    def with_object_type(self, value: str) -> RestapiSubmitReportRequest:
        self.object_type = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "category"):
            result["category"] = str(self.category)
        elif include_empty:
            result["category"] = Union[str, CategoryEnum]()
        if hasattr(self, "reason"):
            result["reason"] = str(self.reason)
        elif include_empty:
            result["reason"] = ""
        if hasattr(self, "user_id"):
            result["userId"] = str(self.user_id)
        elif include_empty:
            result["userId"] = ""
        if hasattr(self, "additional_info"):
            result["additionalInfo"] = {
                str(k0): v0 for k0, v0 in self.additional_info.items()
            }
        elif include_empty:
            result["additionalInfo"] = {}
        if hasattr(self, "comment"):
            result["comment"] = str(self.comment)
        elif include_empty:
            result["comment"] = ""
        if hasattr(self, "extension_category"):
            result["extensionCategory"] = str(self.extension_category)
        elif include_empty:
            result["extensionCategory"] = ""
        if hasattr(self, "object_id"):
            result["objectId"] = str(self.object_id)
        elif include_empty:
            result["objectId"] = ""
        if hasattr(self, "object_type"):
            result["objectType"] = str(self.object_type)
        elif include_empty:
            result["objectType"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        category: Union[str, CategoryEnum],
        reason: str,
        user_id: str,
        additional_info: Optional[Dict[str, Any]] = None,
        comment: Optional[str] = None,
        extension_category: Optional[str] = None,
        object_id: Optional[str] = None,
        object_type: Optional[str] = None,
        **kwargs,
    ) -> RestapiSubmitReportRequest:
        instance = cls()
        instance.category = category
        instance.reason = reason
        instance.user_id = user_id
        if additional_info is not None:
            instance.additional_info = additional_info
        if comment is not None:
            instance.comment = comment
        if extension_category is not None:
            instance.extension_category = extension_category
        if object_id is not None:
            instance.object_id = object_id
        if object_type is not None:
            instance.object_type = object_type
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> RestapiSubmitReportRequest:
        instance = cls()
        if not dict_:
            return instance
        if "category" in dict_ and dict_["category"] is not None:
            instance.category = str(dict_["category"])
        elif include_empty:
            instance.category = Union[str, CategoryEnum]()
        if "reason" in dict_ and dict_["reason"] is not None:
            instance.reason = str(dict_["reason"])
        elif include_empty:
            instance.reason = ""
        if "userId" in dict_ and dict_["userId"] is not None:
            instance.user_id = str(dict_["userId"])
        elif include_empty:
            instance.user_id = ""
        if "additionalInfo" in dict_ and dict_["additionalInfo"] is not None:
            instance.additional_info = {
                str(k0): v0 for k0, v0 in dict_["additionalInfo"].items()
            }
        elif include_empty:
            instance.additional_info = {}
        if "comment" in dict_ and dict_["comment"] is not None:
            instance.comment = str(dict_["comment"])
        elif include_empty:
            instance.comment = ""
        if "extensionCategory" in dict_ and dict_["extensionCategory"] is not None:
            instance.extension_category = str(dict_["extensionCategory"])
        elif include_empty:
            instance.extension_category = ""
        if "objectId" in dict_ and dict_["objectId"] is not None:
            instance.object_id = str(dict_["objectId"])
        elif include_empty:
            instance.object_id = ""
        if "objectType" in dict_ and dict_["objectType"] is not None:
            instance.object_type = str(dict_["objectType"])
        elif include_empty:
            instance.object_type = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, RestapiSubmitReportRequest]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[RestapiSubmitReportRequest]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        RestapiSubmitReportRequest,
        List[RestapiSubmitReportRequest],
        Dict[Any, RestapiSubmitReportRequest],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "category": "category",
            "reason": "reason",
            "userId": "user_id",
            "additionalInfo": "additional_info",
            "comment": "comment",
            "extensionCategory": "extension_category",
            "objectId": "object_id",
            "objectType": "object_type",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "category": True,
            "reason": True,
            "userId": True,
            "additionalInfo": False,
            "comment": False,
            "extensionCategory": False,
            "objectId": False,
            "objectType": False,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "category": ["CHAT", "EXTENSION", "UGC", "USER"],
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the AccelByte Gaming Services Leaderboard Service."""

__version__ = "2.30.3"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .bulk_get_users_ranking__e7e5b3 import BulkGetUsersRankingPublicV3
from .delete_all_user_ranking_204ce6 import DeleteAllUserRankingByCycleIdAdminV3
from .delete_user_ranking_admin_v3 import DeleteUserRankingAdminV3
from .delete_user_ranking_by__2fdfe5 import DeleteUserRankingByCycleIdAdminV3
from .delete_user_ranking_by__c762e5 import DeleteUserRankingByLeaderboardCodeAdminV3
from .delete_user_rankings_admin_v3 import DeleteUserRankingsAdminV3
from .get_all_time_leaderboar_489ddb import GetAllTimeLeaderboardRankingAdminV3
from .get_all_time_leaderboar_5b0d52 import GetAllTimeLeaderboardRankingPublicV3
from .get_current_cycle_leade_f8a4d7 import GetCurrentCycleLeaderboardRankingAdminV3
from .get_current_cycle_leade_2c20ce import GetCurrentCycleLeaderboardRankingPublicV3
from .get_user_ranking_admin_v3 import GetUserRankingAdminV3
from .get_user_ranking_public_v3 import GetUserRankingPublicV3
